Package: MTLR
Type: Package
Title: Survival Prediction with Multi-Task Logistic Regression
Version: 0.2.2
Authors@R: person(given = "Humza",
                  family = "Haider",
                  role = c("aut", "cre"),
                  email = "humzahaider0@gmail.com")
URL: https://github.com/haiderstats/MTLR
BugReports: https://github.com/haiderstats/MTLR/issues
Description: An implementation of Multi-Task Logistic Regression (MTLR) for R. 
  This package is based on the method proposed by Yu et al. (2011) which utilized MTLR for generating individual survival curves
  by learning feature weights which vary across time. This model was further extended to account for left and interval censored data.
License: GPL-2 | file LICENSE
Encoding: UTF-8
Imports: Rcpp (>= 0.12.18), survival (>= 3.0.0), rlang
Suggests: ggplot2 (>= 3.4.0), reshape2 (>= 1.4.3), testthat, vdiffr (>=
        0.3.0), covr, knitr, rmarkdown
Depends: R (>= 3.4.0)
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-13 22:22:08 UTC; Humza.Haider
Author: Humza Haider [aut, cre]
Maintainer: Humza Haider <humzahaider0@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-14 08:40:13 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-14 09:25:24 UTC; unix
Archs: MTLR.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,mtlr)
S3method(predict,mtlr)
S3method(print,mtlr)
export(create_folds)
export(mtlr)
export(mtlr_cv)
export(plotcurves)
importFrom(Rcpp,sourceCpp)
importFrom(rlang,.data)
useDynLib(MTLR)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# MTLR

[![R-CMD-check](https://github.com/haiderstats/MTLR/workflows/R-CMD-check/badge.svg)](https://github.com/haiderstats/MTLR/actions)
[![codecov](https://codecov.io/gh/haiderstats/MTLR/branch/master/graphs/badge.svg)](https://app.codecov.io/gh/haiderstats/MTLR)
![](http://cranlogs.r-pkg.org/badges/grand-total/MTLR)

The goal of `MTLR` is to provide an R implementation for [Multi-Task
Logistic
Regression](https://papers.nips.cc/paper/4210-learning-patient-specific-cancer-survival-distributions-as-a-sequence-of-dependent-regressors).
In addition to supplying the model provided by [Yu et
al.](https://papers.nips.cc/paper/4210-learning-patient-specific-cancer-survival-distributions-as-a-sequence-of-dependent-regressors)
we have extended the model for left censoring, interval censoring, and a
mixture of censoring types. Functionality includes training an MTLR
model, predicting survival curves for new observations, and plotting
these survival curves and feature weights estimated by MTLR.

## Installation

You can install the version from CRAN or the development version from
GitHub:

``` r
# CRAN:
install.packages("MTLR")

# GitHub:
# install.packages("devtools")
devtools::install_github("haiderstats/MTLR")
```

## Example

Given a survival dataset containing event time and event status
indicator (censored/uncensored), we can produce an MTLR model. For
example, consider the `lung` dataset from the `survival` package:

``` r
# Load survival for the lung dataset and the Surv() function.
library(survival)
#Here we will use 9 intervals (10 time points) just for plotting purposes. 
#The default is sqrt the number of observations.
mod <- mtlr(Surv(time,status)~., data = lung, nintervals = 9)
print(mod)
#> 
#> Call:  mtlr(formula = Surv(time, status) ~ ., data = lung, nintervals = 9) 
#> 
#> Time points:
#>  [1]  62.3 145.4 179.3 210.4 241.4 284.5 308.2 383.5 476.3 642.8
#> 
#> 
#> Weights:
#>           Bias      inst      age      sex ph.ecog ph.karno pat.karno meal.cal  wt.loss
#> 62.27   0.1113 -0.017979  0.04894 -0.01250 0.00465 -0.00652   0.01347 -0.02619 -0.01326
#> 145.36  0.1381 -0.021136  0.03275 -0.00471 0.02310 -0.02226  -0.01751 -0.01105 -0.02975
#> 179.27  0.2095 -0.008194  0.02260 -0.02562 0.02393 -0.02045  -0.03160 -0.02310 -0.02218
#> 210.36  0.0464  0.000371  0.00815 -0.03634 0.04702 -0.02227  -0.04127 -0.01410 -0.03367
#> 241.36 -0.1012  0.009570 -0.01580 -0.04404 0.06693 -0.04142  -0.05454 -0.00808 -0.01286
#> 284.55 -0.2372  0.004866 -0.00474 -0.05181 0.04829 -0.02112  -0.03138  0.00237 -0.02549
#> 308.18 -0.2960 -0.007731 -0.00466 -0.05426 0.04912 -0.02284  -0.03266 -0.01607  0.02130
#> 383.45 -0.0286 -0.019974 -0.01029 -0.03263 0.02870 -0.00682  -0.02323 -0.01458 -0.00236
#> 476.27 -0.1277 -0.010224  0.00106 -0.02226 0.01716  0.01699  -0.01556 -0.02111 -0.01589
#> 642.82 -0.3538 -0.014985  0.02291 -0.02550 0.01846  0.00557  -0.00854  0.00376 -0.00555
#Plot feature weights:
plot(mod)
```

<img src="man/figures/README-example-1.png" alt="" style="display: block; margin: auto;" />

``` r
#Get survival curves for the lung dataset:
curves <- predict(mod)
#Plot the first 20 survival curves:
plotcurves(curves, 1:20)
```

<img src="man/figures/README-example-2.png" alt="" style="display: block; margin: auto;" />

#' MTLR Internal Cross-Validation for Selecting C1.
#'
#' @inheritParams mtlr
#' @param C1_vec a vector of regularization parameters to test. All values must be non-negative. For large datasets you may want to reduce the number
#' of value tried to increase efficiency. Similarly for nfolds.
#' @param previous_weights a boolean specifying if sequential folds should use the previous fold's parameters as seed_weights. Doing this will likely
#' speed up the computation time for cross-validation as we are providing weights which are (likely) close to the optimal weights. Note that this is
#' done separately for each value of C1 so there is no parameter sharing between different values of C1, and instead only across the same value of C1.
#' @param loss a string indicating the loss to optimize for which to choose the regularization parameter. Currently one can optimize for the log-likelihood ("ll")
#'  or concordance ("concordance"). See details regarding these losses.
#' @param nfolds the number of internal cross validation folds, default is 5.
#' @param foldtype type of cross validation folds. Full stratification, "fullstrat", sorts observations by their event time and their event indicators
#' and numbers them off into folds. This effectively give each fold approximately the same number of uncensored observations as well as keeps the range
#' of time points as equivalent as possible across folds. This type of cross-validation is completely deterministic.
#' Censored stratification, "censorstrat", will put approximately the same number of uncensored observations in each fold but not pay any attention to
#' event time. This is partially stochastic. The totally random cross-validation, "random", randomly assigns observations to folds without considering
#' event time nor event status.
#' @param verbose if TRUE the progress will be printed for every completed value of C1.
#' @details The log-likelihood loss and concordance are supported for optimizing C1. Here the log-likelihood loss considers censored and uncensored observations differently.
#' For uncensored observations, we assign a loss of the negative log probability assigned to the interval in which the observation had their event, \emph{e.g.}
#' if an observation had a 20% chance of having its event between timepoint1 and timepoint2 and it did have it's event in that interval then the loss
#' is -log(0.2). We want these probabilities to be large so we would normally want to maximize this value (since logs of probabilities are negative)
#' but we take the negative and instead minimize the value, thus we want the lowest loss. For censored observations we take the log of the probability
#' of survival at the time of censoring, \emph{e.g.} if an observation is censored at time = 42 we take the negative log of the survival probability assigned
#' to time 42 as the loss.
#'
#' For the concordance loss, C1 is chosen to maximize the overall concordance when using the negative median as the "risk" score. This is completed using survConcordance in the survival package.
#' @return Performing mtlr_cv will return the following:
#' \itemize{
#'   \item best_C1: The value of C1 which achieved the best (lowest) loss.
#'   \item avg_loss: The averaged value of loss across the five folds for each value of C1 tested.
#'   }
#' @examples
#' library(survival)
#' cv_mod <- mtlr_cv(Surv(time,status)~., data = lung)
#' #Note the best C1 also corresponds to the lost average loss:
#' cv_mod
#' @seealso \code{\link[MTLR]{mtlr}}
#' @export
mtlr_cv <- function(formula,
                 data,
                 time_points = NULL,
                 nintervals = NULL,
                 normalize = T,
                 C1_vec = c(0.001,0.01,0.1,1,10,100,1000),
                 train_biases = T,
                 train_uncensored = T,
                 seed_weights = NULL,
                 previous_weights = T,
                 loss = c("ll","concordance"),
                 nfolds = 5,
                 foldtype = c("fullstrat","censorstrat","random"),
                 verbose = FALSE,
                 threshold = 1e-05,
                 maxit = 5000,
                 lower = -15,
                 upper = 15
                 ){
  if(any(C1_vec < 0)){
    stop("All values of C1 must be non-negative.")
  }
  if(any(dim(data)==0)){
    stop("Dimensions of the dataset must be non-zero.")
  }
  foldtype <- match.arg(foldtype)
  loss <- match.arg(loss)
  lossfnc <- switch(loss,
                    ll = loglik_loss,
                    concordance = concordance_loss
  )
  mf <- stats::model.frame(formula = formula, data)
  y <- stats::model.response(mf)
  time <- y[,1]
  delta <- y[,2]
  fold_index <- create_folds(time,delta,nfolds,foldtype)
  time_delta_names = attr(y,"dimnames")[[2]]
  #There may be some NAs in the time if we are in the inteval setting. We do not want to remove these rows since they
  #are valid NAs. However, we only want x values with nonmissing data.
  data <- data[stats::complete.cases(data[which(!names(data)%in%time_delta_names)]),]
  res_mat <- matrix(rep(0,nfolds*length(C1_vec)), ncol = length(C1_vec),nrow =nfolds)
  #Give the same number of intervals to every fold.
  if(is.null(time_points) & is.null(nintervals)){
    nintervals <- ceiling(sqrt(((nfolds-1)/nfolds)*nrow(y))) #Default number of intervals is the sq. root of the number of observations in k-1/k % of the data.
  }
  if(previous_weights){
    parList <- list()
  }
  for(fold in 1:nfolds){
    if(verbose){
      print(paste("Starting fold ",fold," of ",nfolds,".", sep = ""))
    }
    datacv <- data[-fold_index[[fold]],]
    result <- c()
    for(i in seq_along(C1_vec)){
      if(previous_weights){
        if(fold == 1){
          mod <- mtlr(formula,datacv,time_points,nintervals,
                      normalize, C1_vec[i], train_biases,train_uncensored, seed_weights,
                      threshold, maxit, lower, upper)
          parList[[i]] <- c(mod$weight_matrix)
          time_points = mod$time_points
        }else{
          mod <- mtlr(formula,datacv,time_points,nintervals,
                      normalize, C1_vec[i], train_biases,train_uncensored, parList[[i]],
                      threshold, maxit, lower, upper)
        }
      }else{
        mod <- mtlr(formula,datacv,time_points,nintervals,
                    normalize, C1_vec[i], train_biases,train_uncensored, seed_weights,
                    threshold, maxit, lower, upper)
      }
      result <- c(result, lossfnc(mod,data[fold_index[[fold]],]))
    }
    res_mat[fold,] <- result
  }
  avg_results <- apply(res_mat, 2, mean)
  names(avg_results) <- C1_vec
  best_C1 <- C1_vec[which.min(avg_results)]
  if(loss == "concordance"){
    avg_results <- -1*avg_results
  }
  to_return <- list(best_C1 = best_C1, avg_loss = avg_results)
  return(to_return)
}

testthat::context("Testing the mtlr_cv function. Most tests are just checking against stored values.")



# mtlr_cv ------------------------------------------------------------
testthat::test_that("mtlr_cv function is consistent for basic survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr_cv(formula,data),"mtlrcv_leuk.rds", tolerance = 1e-3)
})

testthat::test_that("mtlr_cv function is consistent when ignoring previous weights",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr_cv(formula,data,previous_weights = F),"mtlrcv_leukNoPrev.rds", tolerance = 1e-3)
})

testthat::test_that("mtlr_cv function is consistent for concordance loss",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr_cv(formula,data, loss = "concordance"),"mtlrcv_leukConcordance.rds", tolerance = 1e-3)
})


testthat::test_that("mtlr_cv function is consistent for more complex survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  skip_on_cran()
  skip_on_travis()
  expect_equal_to_reference(mtlr_cv(formula,data),"mtlrcv_lung.rds", tolerance = 1e-3)
  data$status <- 0
  expect_equal_to_reference(mtlr_cv(formula,data),"mtlrcv_censored.rds", tolerance = 1e-3)
  data$status <- 1
  expect_equal_to_reference(mtlr_cv(formula,data),"mtlrcv_uncensored.rds", tolerance = 1e-3)
})

testthat::test_that("mtlr_cv function catches errors survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  data <- data[data$status == 1,]
  C1_vec <- c(-1,1)
  expect_error(mtlr_cv(formula,data,C1_vec = C1_vec),"All values of C1 must be non-negative.")
  expect_error(mtlr_cv(formula,data.frame()),"Dimensions of the dataset must be non-zero.")
})


testthat::test_that("mtlr_cv function works with multiple types of censoring",{
  time1 <- c(NA, 4, 7, 12, 10, 6, NA, 3,5,9,10,12,NA,4,6,2,NA,16,15,11)
  time2 <- c(14, 4, 10, 12, NA, 9, 5, NA, NA, NA, NA, 15,22,4,8,6,2,20,23,11)
  set.seed(42)
  dat <- cbind.data.frame(time1, time2, importantfeature1 = rnorm(20),importantfeature2 = rnorm(20),
                         importantfeature3 = rnorm(20),importantfeature4 = rnorm(20),importantfeature5 = rnorm(20),
                         importantfeature6 = rbinom(20,1,.3),importantfeature7 = rbinom(20,1,.3))
  formula <- survival::Surv(time1,time2,type = "interval2")~.
  skip_on_cran()
  skip_on_travis()
  expect_equal_to_reference(mtlr_cv(formula, dat, C1_vec = c(.001,.01,.1,0), previous_weights=F ),"mtlrcv_mixed_censoring.rds", tolerance = 1e-3)
})

#' Create folds for cross-validation.
#'
#' Create the test folds for k-fold cross validation. These cross-validation types differ from typical stratified cross-validation as this function also
#' considers the range of event times in the data.
#' @param time a vector of event times.
#' @param delta a vector of indicators for uncensored/censored data. The type of censoring here is not considered so it is suggested this function not
#' be used for data with mixed censoring types. The specific indicator value does not matter as long as censored and uncensored observations have different
#' values for their indicator.
#' @param nfolds The number of folds to create.
#' @param foldtype type of cross validation folds. Full stratification, "fullstrat", sorts observations by their event time and their event indicators
#' and numbers them off into folds. This effectively give each fold approximately the same number of uncensored observations as well as keeps the range
#' of time points as equivalent as possible across folds. This type of cross-validation is completely deterministic.
#' Censored stratification, "censorstrat", will put approximately the same number of uncensored observations in each fold but not pay any attention to
#' event time. This is partially stochastic. The totally random cross-validation, "random", randomly assigns observations to folds without considering
#' event time nor event status.
#' @return a list of size nfolds where each list component contains the indices of the test data for each fold.
#' @seealso \code{\link[MTLR]{mtlr_cv}}
#' @export
create_folds <- function(time, delta, nfolds, foldtype = c("fullstrat","censorstrat","random")){
  if(nfolds < 1)
    stop("Number of folds must be greater than 0.")
  type <- match.arg(foldtype)
  foldIndex <- switch(type,
                      fullstrat = {
                        Order<- order(delta,time)
                        lapply(1:nfolds, function(x) Order[seq(x,length(time), by = nfolds)])
                      },
                      censorstrat = {
                        censored <- sample(which(!delta))
                        uncensored <- sample(which(!!delta))
                        Order<- c(censored,uncensored)
                        lapply(1:nfolds, function(x) Order[seq(x,length(time), by = nfolds)])
                      },
                      random = {
                        Order <- sample(length(time))
                        lapply(1:nfolds, function(x) Order[seq(x,length(time), by = nfolds)])
                      }
  )
  return(foldIndex)
}

testthat::context("Testing the cross-validation fold creator.")

testthat::test_that("create_folds full stratification correctly orders observations.",{
  time <- c(1,2,3,4,5,6,7,8,9,10)
  delta <- c(1,0,1,0,1,0,1,0,1,0)
  foldtype <- "fullstrat"

  expect_equal(create_folds(time, delta, 5, foldtype),
               list(c(2,1),c(4,3),c(6,5),c(8,7),c(10,9)))
  expect_equal(create_folds(time, delta, 2, foldtype),
               list(c(2,6,10,3,7),c(4,8,1,5,9)))
  expect_equal(create_folds(time, delta, 10,foldtype),
               list(2,4,6,8,10,1,3,5,7,9))

  delta <- sort(delta)

  expect_equal(create_folds(time, delta, 5, foldtype),
               list(c(1,6),c(2,7),c(3,8),c(4,9),c(5,10)))
  expect_equal(create_folds(time, delta, 2,foldtype),
               list(c(1,3,5,7,9),c(2,4,6,8,10)))
})

testthat::test_that("create_folds censor stratification correctly stratifies the number of censored observations.",{
  time <- c(1,2,3,4,5,6,7,8,9,10)
  delta <- c(1,0,1,0,1,0,1,0,1,0)
  foldtype <- "censorstrat"

  #Get the number of censored individuals in each fold.
  get_deltas <- function(folds, delta, time){
    return(sapply(folds, function(x) sum(!delta[match(x,time)])))
  }

  expect_equal(get_deltas(create_folds(time, delta, 5, foldtype), delta, time),
               c(1,1,1,1,1))
  expect_equal(get_deltas(create_folds(time, delta, 2, foldtype), delta, time),
               c(3,2))
  expect_equal(get_deltas(create_folds(time, delta, 3, foldtype), delta, time),
               c(2,2,1))
  expect_equal(get_deltas(create_folds(time, delta, 4, foldtype), delta, time),
               c(2,1,1,1))
  expect_equal(get_deltas(create_folds(time, delta, 10, foldtype), delta, time),
               c(1,1,1,1,1,0,0,0,0,0))

  time <- c(1,2,3,4,5)
  delta <- c(1,1,0,0,0)

  expect_equal(get_deltas(create_folds(time, delta, 3, foldtype), delta, time),
               c(1,1,1))
  expect_equal(get_deltas(create_folds(time, delta, 2, foldtype), delta, time),
               c(2,1))
  expect_equal(get_deltas(create_folds(time, delta, 1, foldtype), delta, time),
               c(3))
})


testthat::test_that("create_folds no stratification places an equal(ish) number of observations in each fold.",{
  time <- c(1,2,3,4,5,6,7,8,9,10)
  delta <- c(1,0,1,0,1,0,1,0,1,0)
  foldtype <- "random"

  get_sizes <- function(folds){
    return(sapply(folds, function(x) length(x)))
  }

  expect_equal(get_sizes(create_folds(time,delta,5,foldtype)),
               c(2,2,2,2,2))
  expect_equal(get_sizes(create_folds(time,delta,2,foldtype)),
               c(5,5))
  expect_equal(get_sizes(create_folds(time,delta,1,foldtype)),
               c(10))
  expect_equal(get_sizes(create_folds(time,delta,10,foldtype)),
               c(1,1,1,1,1,1,1,1,1,1))
})

testthat::test_that("create_folds catches incorrect argument format.",{
    expect_error(create_folds(c(1,2,3),c(0,0,1),0,"fullstrat"), "Number of folds must be greater than 0.")
    expect_error(create_folds(c(1,2,3),c(0,0,1),0,"banana"))
})

#' @useDynLib MTLR
#' @importFrom Rcpp sourceCpp
NULL

#' Train a Multi-Task Logistic Regression (MTLR) Model
#'
#' Trains a MTLR model for survival prediction. Right, left, and interval censored data are all supported.
#'
#' @param formula a formula object with the response to the left of the "~" operator. The response must be a survival object returned
#' by the \code{\link[survival]{Surv}} function.
#' @param data a data.frame containing the features for survival prediction. These must be variables corresponding to the formula object.
#' @param time_points the time points for MTLR to create weights. If left as NULL, the time_points chosen will be based on equally spaced quantiles
#' of the survival times. In the case of interval censored data note that only the start time is considered and not the end time for selecting time points.
#' It is strongly recommended to specify time points if your data is heavily interval censored. If time_points is not NULL then nintervals is ignored.
#' @param nintervals Number of time intervals to use for MTLR. Note the number of time points will be nintervals + 1. If left as NULL
#' a default of sqrt(N) is used where N is the number of observations in the supplied dataset. This parameter is ignored if time_points is specified.
#' @param normalize if TRUE, variables will be normalized (mean 0, standard deviation of 1). This is STRONGLY suggested. If normalization
#' does not occur it is much more likely that MTLR will fail to converge. Additionally, if FALSE consider adjusting "lower" and "upper"
#' used for L-BFGS-B optimization.
#' @param C1 The L2 regularization parameter for MTLR. C1 can also be selected via \code{\link[MTLR]{mtlr_cv}}. See "Learning Patient-Specific Cancer Survival Distributions as a Sequence of Dependent
#'  Regressors" by Yu et al. (2011) for details.
#' @param train_biases if TRUE, biases will be trained before feature weights (and again trained while training feature weights). This
#' has shown to speed up total training time.
#' @param train_uncensored if TRUE, one round of training will occur assuming all event times are uncensored. This is done due to the non-convexity issue
#' that arises in the presence of censored data. However if ALL data is censored we recommend setting this option to FALSE as it has shown to give poor
#' results in this case.
#' @param seed_weights the initialization weights for the biases and the features. If left as NULL all weights are initialized to zero. If seed_weights are
#' specified then either nintervals or time_points must also be specified. The length of seed_weights should correspond to (number of features + 1)*(length of
#' time_points) = (number of features + 1)*(nintervals + 1).
#' @param threshold The threshold for the convergence tolerance (in the objective function) when training the feature weights.
#'  This threshold will be passed to \link[stats]{optim}.
#' @param maxit The maximum iterations to run for MTLR. This parameter will be passed to \link[stats]{optim}.
#' @param lower The lower bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.
#' @param upper The upper bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.
#' @details This function allows one to train an MTLR model given a dataset containing survival data. mtlr uses the Limited-Memory
#' Broyden–Fletcher–Goldfarb–Shanno (L-BFGS-B) approximation method to train feature weights. This training is outsourced to the internal
#' \link[stats]{optim} function in R. Currently only a few parameters (namely threshold, maxit,lower, upper) of optim are supported, more will
#' likely become available in the future.
#'
#' Weights are initialized to 0 prior to training. Under default settings, the bias weights
#' will be trained before considering feature weights. As Yu et al. (2011) specified, the introduction of censored observations creates a non-convex
#' loss function. To address this, weights are first trained assuming all event times are \emph{uncensored}. Once these starting weights have
#' been trained another round of training is performed using the true values of the event indicator (censored/uncensored). However, in the event of
#' all censored data this has shown to negatively effect the results. If all data is censored (either left, right, or interval2) we suggest setting
#' train_uncensored = FALSE.
#'
#' Yu et al. (2011) actually suggested two regularization parameters, C1 to control the size of the feature weights and C2 to control the smoothness.
#' In Ping Jin's masters thesis (Using Survival Prediction Techniques to Learn Consumer-Specific Reservation Price Distributions) he showed that C2
#' is not required for smoothness and C1 will suffice (Appendix A.2) so we do not support the C2 parameter in this implementation.
#'
#' If an error occurs from optim it is likely the weights are getting too large. Including fewer time points (or specifying better time points) in
#' addition to changing the lower/upper bounds of L-BFGS-B may resolve these issues. The most common failure has been that the objective value sees
#' infinite values due to extremely large feature weights.
#'
#'\strong{Censored data:} Right, left, and interval censored data are all supported both separately and mixed. The convention to input these types of
#'data follows the \link[survival]{Surv} object format.
#'Per the Surv documentation, "The [interval2] approach is to think of each observation as a time interval with (-infinity, t) for left censored,
#'(t, infinity) for right censored, (t,t) for exact and (t1, t2) for an interval. This is the approach used for type = interval2.
#'Infinite values can be represented either by actual infinity (Inf) or NA." See the examples below for an example of inputting this type of data.
#' @return An mtlr object returns the following:
#' \itemize{
#'   \item weight_matrix: The matrix of feature weights determined by MTLR.
#'   \item x: The dataframe of features (response removed). Note observations with missing values will have been removed (this is the dataset on which
#'   MTLR was trained).
#'   \item y: The matrix of response values MTLR uses for training. Each column corresponds to an observation and rows as time points. A value of 1
#'   indicates a observation was either censored or had their event occur by that time.
#'   \item response: The response as a Surv object (specified by formula).
#'   \item time_points: The timepoints selected and used to train MTLR.
#'   \item C1: The regularization parameter used.
#'   \item Call: The original call to mtlr.
#'   \item Terms: The x-value terms used in mtlr. These are later used in \code{\link[MTLR]{predict.mtlr}}
#'   \item scale: The means and standard deviations of features when normalize = TRUE. These are used in \code{\link[MTLR]{predict.mtlr}}. Will be
#'   NULL if normalize = FALSE.
#'   \item xlevels: The levels of the features used. This is used again by \code{\link[MTLR]{predict.mtlr}}.
#' }
#' @examples
#' #Access the Surv function and the leukemia/lung dataset.
#' library(survival)
#' simple_mod <- mtlr(Surv(time,status)~., data = leukemia)
#' simple_mod
#'
#' bigger_mod <- mtlr(Surv(time,status)~., data = lung)
#' bigger_mod
#'
#' #Note that observations with missing data were removed:
#' nrow(lung)
#' nrow(bigger_mod$x)
#'
#'
#' # Mixed censoring types
#' time1 = c(NA, 4, 7, 12, 10, 6, NA, 3) #NA for right censored
#' time2 = c(14, 4, 10, 12, NA, 9, 5, NA) #NA for left censored
#' #time1 == time2 indicates an exact death time. time2> time1 indicates interval censored.
#' set.seed(42)
#' dat = cbind.data.frame(time1, time2, importantfeature = rnorm(8))
#' formula = Surv(time1,time2,type = "interval2")~.
#' mixedmod = mtlr(formula, dat)
#'
#' @seealso
#' \code{\link[MTLR]{predict.mtlr}} \code{\link[MTLR]{mtlr_cv}} \code{\link[MTLR]{plot.mtlr}}  \code{\link[MTLR]{plotcurves}}
#' @export
mtlr <- function(formula,
                 data,
                 time_points = NULL,
                 nintervals = NULL,
                 normalize = T,
                 C1 = 1,
                 train_biases = T,
                 train_uncensored =T,
                 seed_weights = NULL,
                 threshold = 1e-05,
                 maxit = 5000,
                 lower = -15,
                 upper = 15){
  cl <- match.call() #Save a copy of the function call.
  if(any(dim(data)==0)){
    stop("Dimensions of the dataset must be non-zero.")
  }
  #Data setup
  mf <- stats::model.frame(formula = formula, data)
  Terms <- attr(mf, "terms")
  xlevels <- stats::.getXlevels(Terms, mf)

  x <- stats::model.matrix(Terms, data = mf)
  x <- x[,-1,drop=FALSE] #Remove intercept term -- We will handle biases later.
  y <- stats::model.response(mf)

  if (!survival::is.Surv(y))
    stop("The response must be a Surv object.")
  type = attr(y,"type")
  if(!type %in% c("right", "left","interval","interval2"))
    stop("Currently only right, left,interval, and interval2 censored data is supported. See the Details section of ?mtlr.")

  time <- y[,1]
  if(any(time < 0))
    stop("All event times must be non-negative.")
  if(type %in% c("interval","interval2")){
    time2 = y[,2]
    delta <- y[,3] #Death indicator (right censored = 0, death = 1, left censored = 2, interval censored = 3)
  }else{
    delta <- y[,2] #Death indicator (right censored = 0, death = 1, left censored = 2, interval censored = 3)
  }
  if(C1 < 0)
    stop("C1 must be non-negative.")
  if(threshold <= 0)
    stop("The threshold must be positive.")
  #Extra checks for when seed_weights is not NULL.
  if(!is.null(seed_weights)){
    if(is.null(time_points) & is.null(nintervals))
      stop("When using 'seed_weights' one of 'time_points' or 'nintervals' must also be specified")
    if(!is.null(time_points)){
      if(length(seed_weights) != (ncol(x)+1)*length(time_points))
        stop("The length of seed_weights must equal the product of the (number of features + 1) and the length of time_points.")
    }
    else if(!is.null(nintervals))
      if(length(seed_weights) != (ncol(x)+1)*(nintervals +1))
        stop("The length of seed_weights must equal the product of the (number of features + 1) and (nintervals + 1)")
  }
  if(is.null(nintervals) & is.null(time_points))
    nintervals <- ceiling(sqrt(nrow(y))) #Default number of intervals is the sq. root of the number of observations.
  if(normalize){
    x <- scale(x)
    scale_centers <- attr(x, "scaled:center")
    scale_scale <- attr(x,"scaled:scale")
    scales <- list(center= scale_centers, sd = scale_scale)
    if(any(scale_scale ==0)){
      zeroVar = names(scale_scale)[which(scale_scale ==0)]
      warning(paste("The feature(s): ", paste(zeroVar, collapse = " "), " have zero variance. These features have been set to zero to ensure a feature weight of zero."))
      x[,which(scale_scale ==0)] = 0
    }
  }else{
    scales <- NULL
  }

  ##############################################################
  #Prepare data for MTLR Rcpp functions.
  #The Rcpp functions are built to work on data that comes in with observations ordered by their event status
  #(censored observations first).
  #Here we order our data by the censor status. However, since left and interval censored are coded as 2,3
  #we make these negative first.
  delta = ifelse(delta == 1,1,-delta)
  ord <- order(delta)
  if(ncol(x)){
    x <- x[ord,1:ncol(x),drop=FALSE]
  }

  if(!is.null(nintervals) & is.null(time_points)){
    m <- nintervals + 1   #The number of time points to evaulate will be the number of time intervals + 1.
    quantiles <- seq(0,1,length.out = m+2)[-c(1,m+2)] #We will select time point based on the distribution of the times.
    time_points <- unname(stats::quantile(time, quantiles))
  } #Otherwise time_points is not null and we use the user inputted values
  if(!is.numeric(time_points))
    stop("time_points must be a numeric vector")



  time_points <- time_points[!duplicated(time_points)]#Small data (or common event times) we will have duplicated time points -- we remove them.

  #We make a matrix where each column is a vector of indicators if an observation is dead at each time point, e.g. (0,0,,...,0,1,1,...1).
  #(See 'Learning Patient-Specific Cancer Survival Distributions as a Sequence of Dependent Regressors' Page 3.)
  if(type =="right"){
    y_right <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 0] < c(time_points,Inf)[ind], seq_along(c(time_points,Inf))))),
                      nrow = length(time_points)+1, byrow = T)
    y_uncen <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 1] < c(time_points,Inf)[ind], seq_along(c(time_points,Inf))))),
                      nrow = length(time_points)+1, byrow = T)
    y_matrix <- cbind(y_right, y_uncen)
  } else if(type == "left"){
    y_left <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 0] >= c(0,time_points)[ind],
                                             seq_along(c(time_points,Inf))))), nrow = length(time_points) +1, byrow = T)
    y_uncen <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 1] < c(time_points,Inf)[ind], seq_along(c(time_points,Inf))))),
                      nrow = length(time_points)+1, byrow = T)
    y_matrix <- cbind(y_left,y_uncen)
  } else{
    y_int <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == -3] <= c(time_points,Inf)[ind] & time2[delta == -3] > c(0,time_points)[ind],
                                            seq_along(c(time_points,Inf))))), nrow = length(time_points)+1, byrow = T)
    y_left <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == -2] >= c(0,time_points)[ind],
                                             seq_along(c(time_points,Inf))))), nrow = length(time_points) +1, byrow = T)
    y_right <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 0] < c(time_points,Inf)[ind], seq_along(c(time_points,Inf))))),
                      nrow = length(time_points)+1, byrow = T)
    y_uncen <- matrix(as.numeric(Reduce(c,Map(function(ind) time[delta == 1] < c(time_points,Inf)[ind], seq_along(c(time_points,Inf))))),
                      nrow = length(time_points)+1, byrow = T)
    y_matrix <- cbind(y_int, y_left, y_right, y_uncen)
  }

  #We first train the biases (by setting feature parameters to zero (dAsZero)). Then we train the feature values as if all patients
  #were uncensored (this creates "good" starting values since with censored patients the objective is non-convex). Then we finally
  #train the data with their true censor statuses.
  threshold_factor <- threshold/.Machine$double.eps
  censInd = ifelse(delta <1,0,1)
  if(is.null(seed_weights)){
    if(train_biases){
      zero_matrix <- matrix(0,ncol = ncol(x), nrow = nrow(x))   #We create a zero_matrix to train the biases

      bias_par <- stats::optim(par = rep(0,length(time_points)*(ncol(x) +1)),fn = mtlr_objVal,gr = mtlr_grad, yval = y_matrix,
                               featureValue = zero_matrix, C1=C1, delta = rep(1,nrow(x)),
                               method = "L-BFGS-B", lower = lower, upper = upper, control = c(maxit = maxit, factr = threshold_factor))
      if(bias_par$convergence == 52)
        stop(paste("Error occured while training MTLR. Optim Error: ", bias_par$message))
    }else{
      bias_par <- list(par = rep(0,length(time_points)*(ncol(x) +1)))
    }
    if(train_uncensored){
      params_uncensored <- stats::optim(par = bias_par$par,fn = mtlr_objVal, gr = mtlr_grad, yval = y_matrix, featureValue = x, C1 = C1, delta = rep(1,nrow(x)),
                                        method = "L-BFGS-B", lower = lower, upper = upper, control = c(maxit = maxit, factr = threshold_factor))
      if(params_uncensored$convergence == 52)
      stop(paste("Error occured while training MTLR. Optim Error: ", params_uncensored$message))
    }else{
      params_uncensored = bias_par
    }

    final_params <- stats::optim(par = params_uncensored$par,fn = mtlr_objVal,gr = mtlr_grad, yval = y_matrix, featureValue = x, C1 = C1,delta = sort(censInd),
                                 method = "L-BFGS-B", lower = lower, upper = upper, control = c(maxit = maxit, factr = threshold_factor))
    if(final_params$convergence == 52)
      stop(paste("Error occured while training MTLR. Optim Error: ", final_params$message))
  } else{
    final_params <- stats::optim(par = seed_weights,fn = mtlr_objVal,gr = mtlr_grad, yval = y_matrix, featureValue = x, C1 = C1,delta = sort(censInd),
                                 method = "L-BFGS-B", lower = lower, upper = upper, control = c(maxit = maxit, factr = threshold_factor))
  }


  weights <- matrix(final_params$par, ncol = ncol(x) + 1,byrow=FALSE)
  colnames(weights) <- c("Bias",colnames(x))
  rownames(weights) <- round(time_points,2)
  x <- x[order(ord),0:ncol(x),drop = FALSE]
  y_matrix <- y_matrix[,order(ord)]
  fit <- list(weight_matrix = weights,
              x = x,
              y = y_matrix,
              response = y,
              time_points = time_points,
              C1 = C1,
              Call = cl,
              Terms = Terms,
              scale = scales,
              xlevels = xlevels)
  class(fit) <- "mtlr"
  fit
}

testthat::context("Testing the primary mtlr function.")



# mtlr ------------------------------------------------------------
testthat::test_that("mtlr function is consistent for basic survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr(formula,data),"mtlr_leuk.rds")
})

testthat::test_that("mtlr function doesn't fail for 0 varaince features",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  data$x <- 1
  expect_warning(mtlr(formula,data))
})

testthat::test_that("mtlr function is consistent for more complex survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  expect_equal_to_reference(mtlr(formula,data),"mtlr_lung.rds")
})

testthat::test_that("mtlr function is consistent for more complex survival dataset - no extra bias training",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  expect_equal_to_reference(mtlr(formula,data, train_biases = F),"mtlr_lung_nobias.rds")
})

testthat::test_that("mtlr function is consistent for all censored survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  data <- data[data$status == 0,]
  expect_equal_to_reference(mtlr(formula,data),"mtlr_censored.rds")
})

testthat::test_that("mtlr function is consistent for all uncensored survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  data <- data[data$status == 1,]
  expect_equal_to_reference(mtlr(formula,data),"mtlr_uncensored.rds")
})

testthat::test_that("mtlr function is consistent for basic survival dataset UNNORMALIZED",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr(formula,data, normalize = F),"mtlr_leukUNNORMALIZED.rds")
})

testthat::test_that("mtlr function is consistent for basic survival dataset for chosen nintervals",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  expect_equal_to_reference(mtlr(formula,data, nintervals = 3),"mtlr_leuk_timepoints.rds")
})

testthat::test_that("mtlr function works with left censoring",{
  formula <- survival::Surv(time,status, type = "left")~.
  data <- survival::lung
  expect_equal_to_reference(mtlr(formula,data),"mtlr_leuk_left.rds")
})

testthat::test_that("mtlr function works with multiple types of censoring",{
  time1 <- c(NA, 4, 7, 12, 10, 6, NA, 3,5,9,10,12,NA,4,6,2,NA,16,15,11)
  time2 <- c(14, 4, 10, 12, NA, 9, 5, NA, NA, NA, NA, 15,22,4,8,6,2,20,23,11)
  set.seed(42)
  dat <- cbind.data.frame(time1, time2, importantfeature1 = rnorm(20),importantfeature2 = rnorm(20),
                         importantfeature3 = rnorm(20),importantfeature4 = rnorm(20),importantfeature5 = rnorm(20),
                         importantfeature6 = rbinom(20,1,.3),importantfeature7 = rbinom(20,1,.3))
  formula <- survival::Surv(time1,time2,type = "interval2")~.
  expect_equal_to_reference(mtlr(formula, dat),"mtlr_mixed_censoring.rds", tolerance = 1e-3)
})



testthat::test_that("mtlr argument specifications are working.",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  data$time[1] <- -10
  expect_error(mtlr(formula,data),"All event times must be non-negative")
  data$time[1] <- 10

  formula <- time~.
  expect_error(mtlr(formula,data),"The response must be a Surv object.")

  formula <- survival::Surv(time,status)~.

  expect_error(mtlr(formula,data, C1 = -10),"C1 must be non-negative.")
  expect_error(mtlr(formula,data, C1 = -1e-10),"C1 must be non-negative.")
  expect_error(mtlr(formula,data, threshold = -1e-10),"The threshold must be positive.")
  expect_error(mtlr(formula,data, threshold = 0),"The threshold must be positive.")
  expect_error(mtlr(formula,data.frame()),"Dimensions of the dataset must be non-zero.")

})

testthat::test_that("when training mtlr fails optim error is caught",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  data$meal.cal <- data$meal.cal*1e100
  expect_error(mtlr(formula,data,normalize = F))

})

testthat::context("Testing methods for mtlr objects.")


#Here we will write very basic tests for print and plot since these
#objects are hard to write tests for. Additionally, predict is subtly
#checked in other functions by requiring the output to be consistent.
#however we add a few more tests.

# mtlr ------------------------------------------------------------
testthat::test_that("print and plot functions are consistent for basic survival dataset",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  mod <- mtlr(formula, data)
  expect_equal_to_reference(print(mod),"print.rds")
  basePlot <- plot(mod)
  expect_doppelganger("basePlot", basePlot)

  data <- survival::leukemia
  mod <- mtlr(formula, data)
  expect_warning(plot(mod))
})


testthat::test_that("predict function works across the different types",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  mod <- mtlr(formula, data)
  expect_equal_to_reference(predict(mod,data, type = "prob_event"),"predProb.rds")
  expect_equal_to_reference(predict(mod,data, type = "mean_time"),"predMean.rds" )
  expect_equal_to_reference(predict(mod,data, type = "median_time"),"predMedian.rds" )

  expect_error(predict(mod,survival::leukemia))

  expect_equal_to_reference(predict(mod,data,type = "survivalcurve", add_zero = F),"curve_NoZero.rds")
  expect_equal_to_reference(predict(mod,data,type = "prob_event", add_zero = F),"predProb_NoZero.rds")
  expect_equal_to_reference(predict(mod,data,type = "mean_time", add_zero = F),"predMean_NoZero.rds")
  expect_equal_to_reference(predict(mod,data, type = "median_time", add_zero = F),"predMedian_NoZero.rds")

})

testthat::context("plotcurves is consistent with previous plots.")



# plotcurves ------------------------------------------------------------
testthat::test_that("plotcurves function is consistent",{
  formula <- survival::Surv(time,status)~.
  data <- survival::lung
  mod <- mtlr(formula,data)
  curves <- predict(mod)
  expect_warning(plotcurves(curves, index=  1:3, color = c("red","green")))
  oneCurve <- plotcurves(curves)
  expect_doppelganger("oneCurve", oneCurve)
  onePinkCurve <- plotcurves(curves, color = "pink")
  expect_doppelganger("onePinkCurve", onePinkCurve)
  manyCurves <- plotcurves(curves, 1:10)
  expect_doppelganger("manyCurves", manyCurves)
  shortCurve <- plotcurves(curves, xlim = c(0,4))
  expect_doppelganger("shortCurve", shortCurve)
})

#' Printing an MTLR object.
#'
#' Print an object created by \code{\link[MTLR]{mtlr}}.
#' @param x an object of class mtlr (result from calling \code{\link[MTLR]{mtlr}}).
#' @param digits The number of digits to print mtlr weights.
#' @param ... for future methods.
#' @return Call, the original call to the mtlr function. Time points, the time points selected by the mtlr model. Weights, the weights of
#' each feature across time -- rows represent each time point and each column corresponds to a feature.
#' @seealso \code{\link[MTLR]{mtlr}}
#' @export
print.mtlr <- function(x, digits = max(options()$digits - 4,3), ...){
  cat("\nCall: ", deparse(x$Call), "\n")
  cat("\nTime points:\n")
  print(x$time_points, digits = digits)
  cat("\n\nWeights:\n" )
  print(x$weight_matrix,digits = digits)
}


#' Predictions for MTLR
#'
#' Compute survival curves and other fitted values for a model generated by \code{\link[MTLR]{mtlr}}.
#'
#' @param object an object of class mtlr, generated by the \link[MTLR]{mtlr}.
#' @param newdata an optional new dataframe for which to perform predictions using MTLR. If left empty, predictions will
#' be performed using the dataset used to generate the original mtlr object -- note that any error calculation on these
#' predictions will be optimistic since this will only be the resubstitution error and not be representative of error on a new test set.
#' @param type the type of prediction desired. Options are the survival curve for the time points selected by mtlr ("survivalcurve"), the
#' survival curve for given times ("prob_times"), the probability of survival at the observations event time ("prob_event"),
#' the mean survival time ("mean_time"), and the median survival time ("median_time").
#'
#' For "survivalcurve" and "prob_times", the first column of the matrix returned will correspond to the time points and all other columns will be
#' the observations survival probability at those associated time points. The index of a (row) observation in newdata will correspond
#' to the \emph{ith + 1} column of the returned matrix.
#'
#' If "prob_event" is chosen the response (event time) is required. For both "prob_event" and "prob_times", if the event time is larger
#' than all of the time points used to build the mtlr model then the last (lowest) probability is used. For example, if the event time is
#' 100 but the largest time point estimated by the mtlr model was 80 then the survival probability at 100 is equal to the survival
#' probability at 80, \emph{i.e. S(100) = S(80)}.
#'
#' For "mean_time", if survival curves do not extend to zero survival
#' probability a linear extension is added (a linear line from (time = 0,probability = 1) to (time = ?, probability =0)). This is the
#' same for "median_time" except the line need only extend to survival probability = 0.5.
#' A mean/median survival time of Inf is returned for survival curves with all survival probabilities of 1.
#' @param add_zero if TRUE, a time point of "0" and a survival probability of "1" will be added to all survival curves. Additionally,
#' if add_zero is TRUE, type = "mean_time" will represent the average survival time overall but if FALSE, then "mean_time" will be reduced
#' by roughly the value of the first time point. However, "median_time" and "prob_event" will be unchanged.
#' @param times For prediction method "prob_times" you may specify the times at which to predict the survival probability for each row in newdata.
#' This values defaults to all unique event times (both censored and uncensored) in the data on which the model was trained.
#' @param ... for future methods.
#' @return The desired prediction type (a matrix or vector of predictions).
#' @note The predictions generated by type = "survivalcurve" can be plotted using \code{\link[MTLR]{plotcurves}} -- packages
#' ggplot2 and reshape2  must be installed to use this function.
#' @seealso \code{\link[MTLR]{mtlr}} \code{\link[MTLR]{plotcurves}}
#' @examples
#' library(survival)
#' mod <- mtlr(Surv(time,status)~., data = lung)
#'
#' #Here our predictions are on the data from which we trained so our results will be optimistic
#' # since they are produced from resubstitution as opposed to some new test set.
#' predict(mod, type = "survivalcurve")
#' predict(mod, type = "prob_event")
#' predict(mod, type = "median_time")
#' predict(mod, type = "mean_time")
#'
#' #Notice the difference of about 59:
#' predict(mod, type = "mean_time", add_zero = FALSE)
#' @export
predict.mtlr <- function(object, newdata, type = c("survivalcurve","prob_times","prob_event","mean_time","median_time"),
                         add_zero = T, times = c(),...){
  type <- match.arg(type)
  if(type == "prob_times" & length(times) < 1){
    times = sort(unique(object$response[,1]))
  }
  if(missing(newdata)){
    newframe <- object$x
  }else{
    Terms <- object$Terms
    Terms <- stats::delete.response(Terms)
    newframe <- stats::model.matrix(Terms, data=newdata,
                             xlev=object$xlevels)
    newframe <- newframe[1:nrow(newframe),-1, drop = FALSE] #Remove intercept term.
    if(!is.null(object$scale)){
      newframe <- scale(newframe, center = object$scale$center, scale = object$scale$sd)
      if(any(object$scale$sd == 0)) #If a variable had 0 variance in training set all the values to 0 since they will be NaN right now.
        newframe[,which(object$scale$sd == 0)] = 0
    }
  }

  surv_probs <- mtlr_predict(c(object$weight_matrix), newframe)
  #Issue due to machine precision, we get survival probabilities of 1+e-16. So here we adjust for that.
  surv_probs[surv_probs > 1] <- 1
  if(add_zero){
      times = c(0, times)
      time_points <- c(0,object$time_points)
      surv_probs <- rbind(1,surv_probs)
  }else{
    time_points <- object$time_points
  }
  surv_curves <- cbind.data.frame(time = time_points,surv_probs)
  switch(type,
         survivalcurve = surv_curves,
         prob_times = {
           surv = cbind.data.frame(time = times, sapply(surv_curves[,-1], function(x) predict_prob(x, time_points, times)))
           #We get row names of 'times' so we will remove those.
           row.names(surv) = NULL
           surv
         },
         prob_event = {
           if(!missing(newdata)){
             mf <- stats::model.frame(object$Terms, newdata)
             y <- stats::model.response(mf)
           }else{
             y <- object$response
           }
           event_times <- y[,1]
           unname(mapply(function(x,y) predict_prob(x, time_points,y), surv_curves[,-1], event_times))
         },
         mean_time = {
           unname(sapply(surv_curves[,-1], function(x) predict_mean(x, time_points)))
         },
         median_time = {
           unname(sapply(surv_curves[,-1],function(x) predict_median(x, time_points)))
         }
  )
}

#' Graphical Representation of Feature Weights
#'
#' Plot the weights of an mtlr object. If packages ggplot2 and reshape2 are
#' not installed, a bargraph of feature \emph{influence} is given where influence is defined as the sum of absolute values of the
#' feature weights across time. If ggplot2 and reshape2 are installed then a plot of feature weight across time is given.
#'
#'@param x an object of class mtlr (result from calling \code{\link[MTLR]{mtlr}}).
#'@param numfeatures the number of weight to plot. Default is 5. The most influential features are chosen first.
#'@param featurenames the names of the specific weight to plot. These should correspond to the names
#'in x$weight_matrix. If featurenames are supplied, then numfeatures is ignored.
#'@param digits the number of digits to round to for the value of the time points.
#'@param ... for future methods
#'@examples
#'#These examples are geared towards users who have installed ggplot2 and reshape2.
#'library(survival)
#'mod <- mtlr(Surv(time,status)~., data = lung)
#'#Basic plot with 5 most influential features
#'plot(mod)
#'#Plot all 8 features
#'plot(mod, numfeatures = 8)
#'#Suppose we want to see specifically the "meal.cal" and "ph.karno" features:
#'plot(mod, featurenames = c("meal.cal", "ph.karno"))
#'@export
plot.mtlr <- function(x, numfeatures=5, featurenames = c(), digits, ...) {
  if (requireNamespace(c("ggplot2","reshape2"), quietly = TRUE)) {
    weights <- x$weight_matrix
    time_points <- x$time_points
    if(!length(featurenames)){
      influence <- get_param_influence(x)
      if(numfeatures > length(influence)){
        warning("Number of features specified greater than the total number of features. This has been replaced with the total number of features.")
        numfeatures <- length(influence)
      }
      top_weights <- influence[order(influence, decreasing = T)[1:numfeatures]]
      top_weight_names <- names(top_weights)
      weight_index <- match(top_weight_names, colnames(weights))
    }else{
      weight_index <- match(featurenames, colnames(weights))
    }
    plot_weights <- weights[,weight_index, drop = FALSE]
    chr_time <- as.character(round(time_points,digits))
    plot_data <- cbind.data.frame(time = chr_time, plot_weights)
    plot_data <- reshape2::melt(plot_data,id.vars = "time")
    ggplot2::ggplot(plot_data, ggplot2::aes(x = .data[["time"]],
                                            y = .data[["value"]],
                                            group = .data[["variable"]],
                                            color = .data[["variable"]])) +
      ggplot2::geom_point(size = 2.5) +
      ggplot2::geom_line(linewidth = 1.25) +
      ggplot2::scale_x_discrete("Event Time", limits = chr_time)+
      ggplot2::theme_bw() +
      ggplot2::labs(y = "Weight", color = "Feature") +
      ggplot2::theme(text = ggplot2::element_text(size = 16),
            axis.text = ggplot2::element_text(size = 12, face = "bold"))
  } else {
    warning("Installing ggplot2 and reshape2 will given a more informative plot.")
    influence <- get_param_influence(x)
    graphics::barplot(influence, ylab = "Influence", xlab = "Feature")
  }
}

#' Graphically Visualize MTLR Survival Curves
#'
#' Plot the survival curves returned from predict.mtlr. Users must have packages ggplot2 and reshape2 installed in order to use this function.
#' Survival curves for MTLR are smoothed using a monotonic cubic spline using a Hyman filtering between time points. For details regarding this
#' smoothing function see \code{\link[stats]{splinefun}}.
#'
#' @param curves survival curves formatted the same as those from predict.mtlr. Time points must be in the first column of the matrix followed by
#' columns representing survival probabilities for each observation.
#' @param index the index of the observation to plot. Here an index of 1 will refer to the second column of the curves object. If over 15 indices are
#' given the legend will be removed as to not take up plotting space. To avoid this behavior set remove_legend = FALSE.
#' @param color the color of the plotted survival curve. The length of color must match the length of index.
#' @param xlim the limits of the x-axis (must be a 2 length vector).
#' @param remove_legend if TRUE the legend will be removed if over 15 indices are supplied. If FALSE the legend will remain, however be aware that the
#' legend may take up lots of space.
#' @examples
#' #Set up the example:
#' library(survival)
#' mod <- mtlr(Surv(time,status)~., data = lung)
#' curves <- predict(mod, type = "survivalcurve")
#'
#' plotcurves(curves, 1:10)
#' plotcurves(curves, 1:3, color = c("red","blue","purple"))
#' plotcurves(curves, 1:10, xlim = c(0,42))
#'
#' #Note the legend is now gone:
#' plotcurves(curves, 1:20)
#'
#' #and it is back again:
#' plotcurves(curves, 1:20, remove_legend = FALSE)
#' @seealso \code{\link[MTLR]{mtlr}} \code{\link[MTLR]{predict.mtlr}}
#' @export
plotcurves <- function(curves, index = 1, color = c(), xlim = c(), remove_legend = TRUE){
  if (!requireNamespace(c("ggplot2","reshape2"), quietly = TRUE)) {
    stop("Package \"ggplot2\" and \"reshape2\" are needed for this function to work. Please install them.",
         call. = FALSE)
  }
  colorOK <- T
  if(!length(color))
    colorOK <- F
  else if(length(color) != length(index)){
    warning("If you would like to select custom colors please make sure the number of colors
            matches the number of curves.")
    colorOK <- F
  }
  time <- curves[,1]
  curves <- curves[,index +1,drop=F]
  plot_times <- seq(min(time),max(time), length.out = length(time)*100)
  plot_probs <- as.data.frame(sapply(curves,
                                   function(curve){
                                     curve <- ifelse(curve < 1e-20,0,curve)
                                     survivialSpline <- stats::splinefun(time, curve, method = "hyman")
                                     return(pmax(survivialSpline(plot_times),0))
                                   }
  ))
  plot_data <- cbind.data.frame(plot_times,plot_probs)
  plot_data <- reshape2::melt(plot_data,measure.vars = names(plot_data)[-1], variable.name = "Index")
  pl <- ggplot2::ggplot(data = plot_data, ggplot2::aes(x = .data[["plot_times"]],y = .data[["value"]], colour = .data[["Index"]]))+
    ggplot2::geom_line(linewidth = 1.5)
  if(colorOK)
    pl <- pl + ggplot2::scale_color_manual(values = color)
  if(length(xlim)==2){
    pl <- pl+ ggplot2::coord_cartesian(xlim =c(xlim[1],xlim[2]), ylim= c(0,1))
  } else {
    pl <- pl + ggplot2::coord_cartesian(ylim= c(0,1))
  }
  pl <- pl +
    ggplot2::scale_y_continuous(breaks = c(0,.1,.2,.3,.4,.5,.6,.7,.8,.9,1))+
    ggplot2::theme_bw() +
    ggplot2::theme(
          text = ggplot2::element_text(size=18, face=  "bold"),
          axis.title = ggplot2::element_text(size = 20),
          axis.title.x = ggplot2::element_text(margin = ggplot2::margin(t = 15)),
          axis.title.y = ggplot2::element_text(margin = ggplot2::margin(r = 15)))+
    ggplot2::labs(y = "Survival Probability",x = "Time", color = "Index" )
  #If we have too many survival curves then the legend takes up the whole plot. We have an if to catch this and remove the legend.
  if(length(index) > 15 & remove_legend){
    pl <- pl + ggplot2::theme(legend.position = "None")
  }
  return(pl)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mtlr_objVal <- function(params, yval, featureValue, C1, delta) {
    .Call('_MTLR_mtlr_objVal', PACKAGE = 'MTLR', params, yval, featureValue, C1, delta)
}

mtlr_grad <- function(params, yval, featureValue, C1, delta) {
    .Call('_MTLR_mtlr_grad', PACKAGE = 'MTLR', params, yval, featureValue, C1, delta)
}

mtlr_predict <- function(params, featureValue) {
    .Call('_MTLR_mtlr_predict', PACKAGE = 'MTLR', params, featureValue)
}

#utility.R contains functions that are used by exported functions but are not exported themselves.


#Helper function for plot.mtlr -- Here we get the sum of the absolute values of the feature weights across time.
#This essentially measure how much "influence" each feature had on the survival of each observation.
get_param_influence <- function(mtlr_object){
  weights<- mtlr_object$weight_matrix
  weights <- weights[,-1, drop= FALSE] #Remove bias
  influence <- apply(weights,2,function(x) sum(abs(x)))
  return(influence)
}


#Helper function for predict.mtlr

#We need some type of predict function for survival curves - here we build a spline to fit the survival model curve. This spline is
#the montotone spline using the hyman filtering of the cubic Hermite spline method,
#see https://en.wikipedia.org/wiki/Monotone_cubic_interpolation. Also see help(splinefun).
#Note that for timepoints outside of the fit the spline is constant, i.e. all probabilities outside of the max predicted_times will be constant.
predict_prob <- function(survival_curve,predicted_times, time_to_predict){
  spline <- stats::splinefun(predicted_times, survival_curve, method = "hyman")
  maxTime <- max(predicted_times)
  predictedProbabilities <- rep(0, length(time_to_predict))
  maxProb <- which(time_to_predict > maxTime)
  if(length(maxProb) > 0){
    #We enforce that times outside of the max time are constant.
    predictedProbabilities[maxProb] <- spline(maxTime)
    #If time_to_predict is less than predicted_times then we will enforce this probability to be a max of 1.
    predictedProbabilities[-maxProb] <- pmin(spline(time_to_predict[-maxProb]),1)
  }
  else{
    predictedProbabilities <- pmin(spline(time_to_predict),1)
  }
  return(predictedProbabilities)
}



#We calculate the mean and median survival times assuming a monotone spline fit of the survival curve points.
predict_mean <- function(survival_curve, predicted_times){
  #If all the predicted probabilities are 1 the integral will be infinite. For this reason we slightly decrease the
  #last value.
  if(all(survival_curve==1)){
    return(Inf)
  }
  spline <- stats::splinefun(predicted_times, survival_curve, method = "hyman")
  maxTime <- max(predicted_times)
  slope <- (1-spline(maxTime))/(min(predicted_times) - max(predicted_times))
  zeroProbabilitiyTime <- min( predicted_times[which(survival_curve ==0)], maxTime + (0-spline(maxTime))/slope)
  splineWithLinear <- function(time) ifelse(time < maxTime, spline(time),1 + time*slope)
  area <- stats::integrate(splineWithLinear,min(predicted_times), zeroProbabilitiyTime,subdivisions = 1000,rel.tol = .0001)[[1]]
  return(area)
}


predict_median <- function(survival_curve, predicted_times){
  #If all the predicted probabilities are 1 the median will be infinite.
  if(all(survival_curve==1)){
    return(Inf)
  }
  spline <- stats::splinefun(predicted_times, survival_curve, method = "hyman")
  minProb <- min(spline(predicted_times))
  maxProb <- max(spline(predicted_times))
  if(maxProb < 0.5) return(min(predicted_times))
  if(minProb < 0.5){
    maximumSmallerThanMedian <- predicted_times[min(which(survival_curve <.5))]
    minimumGreaterThanMedian <- predicted_times[max(which(survival_curve >.5))]
    splineInv <- stats::splinefun(spline(seq(minimumGreaterThanMedian, maximumSmallerThanMedian, length.out = 1000)),
                          seq(minimumGreaterThanMedian, maximumSmallerThanMedian, length.out = 1000))
    medianProbabilityTime <- splineInv(0.5)
  } else{
    maxTime <- max(predicted_times)
    slope <- (1-spline(maxTime))/(min(predicted_times) - max(predicted_times))
    medianProbabilityTime <- maxTime + (0.5-spline(maxTime))/slope
  }
  return(medianProbabilityTime)
}

loglik_loss <- function(object, newdata){
  #For the loss we need to compute losses differently for censored and uncensored patients.
  #For right censored patients the loss will correspond the the (log) survival probability assigned by the model at the time of censoring.
  #For left censored patients the loss will correspond the the (log) death probability (1- survival) assigned by the model at the time of censoring.
  #For interval censored patients the loss will correspond the the (log) probability assigned to the interval (probability of survival lower bound time -
  #probability of survival upper bound time).
  #For uncensored patients, we will consider the log of the probability assigned to the time interval when the patient died.
  #Then we take the negative of this loss and thus would like to minimize the loss.

  #Get the survival curves for all observations.
  curves <- predict.mtlr(object,newdata, type = "survivalcurve")


  #We need to know which observations are censored in the new data.
  Terms <- object$Terms
  mf <- stats::model.frame(Terms, data=newdata,xlev=object$xlevels)
  response <- stats::model.response(mf)

  type = attr(response,"type")
  event_times <- response[,1]

  if(type %in% c("interval","interval2")){
    event_times2 = response[,2]
    delta <- response[,3] #Death indicator (right censored = 0, death = 1, left censored = 2, interval censored = 3)
  }else{
    delta <- response[,2] #Death indicator (right censored = 0, death = 1, left censored = 2, interval censored = 3)
  }

  censor_ind <- which(delta!=1)
  uncen_ind <- which(delta==1)

  #Initialize loss to 0.
  logloss <- 0

  #Censored patients
  if(length(censor_ind)){
    if(type == "right" | type == "left"){
      censor_prob <- sapply(censor_ind,
                            function(index) predict_prob(curves[,index+1],
                                                         curves[,1],
                                                         event_times[index]
                            ))
      censor_curves = curves[,(censor_ind +1), drop=F]
      censor_event = event_times[censor_ind]
      censor_prob <- mapply(function(x,y) predict_prob(x, curves[,1],y),
                            censor_curves, censor_event)
      #Currently we add a value of 0.00001 to all values since otherwise we might take the log of 0.
      #This is clearly not ideal but shouldn't make a large impact especially since this is only used for selecting C1.
      if(type == "right"){
        #For right censor we have a loss of the probability of surviving to censor time, log(1) is optimal, log (0) is worst case.
        logloss <- logloss - sum(log(censor_prob + 1e-05))
      }else{
        #For left censor we have a loss of the probability of dying before censor time, log(1) is optimal, log (0) is worst case.
        logloss <- logloss - sum(log(1-censor_prob + 1e-05))
      }
    }else{
      left_right <- which(delta %in% c(0,2))
      interval <- which(delta %in% c(3))


      left_right_curves = curves[,(left_right +1), drop=F]
      left_right_event = event_times[left_right]
      left_right_prob <- mapply(function(x,y) predict_prob(x, curves[,1],y),
                            left_right_curves, left_right_event)

      interval_curves = curves[,(interval +1), drop=F]
      interval_eventL = event_times[interval]
      interval_eventU = event_times2[interval]

      interval_probL <- mapply(function(x,y) predict_prob(x, curves[,1],y),
                               interval_curves, interval_eventL)
      interval_probU <- mapply(function(x,y) predict_prob(x, curves[,1],y),
                               interval_curves, interval_eventU)

      left_right_prob <- ifelse(left_right %in% which(delta==0), left_right_prob, 1 - left_right_prob)
      if(length(interval_probL)){
        interval_prob <- interval_probL - interval_probU
        censor_prob <- c(left_right_prob, interval_prob)
      }else{
        censor_prob = left_right_prob
      }
      logloss <- logloss - sum(log(censor_prob + 1e-05))
    }

  }

  #Uncensored patients
  if(length(uncen_ind)){
    uncen_curves <- curves[,uncen_ind+1,drop=FALSE]
    uncen_curves <- rbind(uncen_curves, 0)
    #Get the probability of the event occuring in every interval.
    pmf_probs <- -apply(uncen_curves,2,diff)

    #This will find the index of which interval the event occured we then index pmf_probs the by observation index
    #and the event index to generate the vector of probabilities assigend to each observations respective event time.
    event_ind <- sapply(event_times[uncen_ind], function(x) findInterval(x, curves[,1]))
    index_col <- 1:length(uncen_ind)
    index_mat <- matrix(c(event_ind,index_col),ncol = 2)
    probs <- pmf_probs[index_mat]
    logloss <- logloss  - sum(log(probs+1e-05))
  }
  return(logloss/nrow(newdata))
}

#' @importFrom rlang .data

concordance_loss <- function(object, newdata){
  #Get the survival curves for all observations.
  preds <- -1*predict.mtlr(object,newdata, type = "median")
  Terms <- object$Terms
  mf <- stats::model.frame(Terms, data=newdata,xlev=object$xlevels)
  response <- stats::model.response(mf)
  conc <- -1*unname(survival::concordance(response~I(-preds))$concordance)
  return(conc)
}

testthat::context("Testing the additional methods used by the predict function (predict_prob, predict_mean, predict_median).")

#We test these prediction methods with (mostly) a linear survival curve because it is the easiest to understand.


# predict_prob ------------------------------------------------------------
testthat::test_that("predict_prob spline functionality",{
  survival_curve <- seq(1,0,length.out = 101)
  predicted_times <- 0:100

  expect_equal(predict_prob(survival_curve, predicted_times, 0.5), 0.995)
  expect_equal(predict_prob(survival_curve, predicted_times, 49.5), 0.505)
  expect_equal(predict_prob(survival_curve, predicted_times, 50), 0.50)
  expect_equal(predict_prob(survival_curve, predicted_times, 99), 0.01)
  expect_equal(predict_prob(survival_curve, predicted_times, 99.5), 0.005)
})

testthat::test_that("predict_prob flat survival curve",{
  survival_curve <- seq(1,0,length.out = 101)
  predicted_times <- 0:100

  expect_equal(predict_prob(survival_curve, predicted_times, 0), 1.00)
  expect_equal(predict_prob(survival_curve, predicted_times, -10), 1.00)
  expect_equal(predict_prob(survival_curve, predicted_times, 100), 0.00)
  expect_equal(predict_prob(survival_curve, predicted_times, 150), 0)

  survival_curve <- rep(1,101)
  predicted_times <- 0:100

  expect_equal(predict_prob(survival_curve, predicted_times, 0), 1.00)
  expect_equal(predict_prob(survival_curve, predicted_times, 100), 1.00)
  expect_equal(predict_prob(survival_curve, predicted_times, 150), 1.00)
  expect_equal(predict_prob(survival_curve, predicted_times, 1000), 1.00)
})

testthat::test_that("predict_prob constant extension functionality",{
  survival_curve <- seq(1,.5,length.out = 51)
  predicted_times <- 0:50

  expect_equal(predict_prob(survival_curve, predicted_times, 51), 0.5)
  expect_equal(predict_prob(survival_curve, predicted_times, 75), 0.5)
  expect_equal(predict_prob(survival_curve, predicted_times, 100), 0.5)
  expect_equal(predict_prob(survival_curve, predicted_times, 150), 0.5)
})









# predict_mean ------------------------------------------------------------
testthat::test_that("predict_mean functionality",{
  #Full linear survival curve
  survival_curve <- seq(1,0,length.out = 101)
  predicted_times <- 0:100
  expect_equal(predict_mean(survival_curve, predicted_times), 50)

  #Linear extension
  survival_curve <- seq(1,.5,length.out = 51)
  predicted_times <- 0:50
  expect_equal(predict_mean(survival_curve, predicted_times), 50)

  #Infinite survival curve
  survival_curve <- rep(1,101)
  predicted_times <- 0:100
  expect_equal(predict_mean(survival_curve, predicted_times), Inf)

  #Zero survival curve
  survival_curve <- rep(0,101)
  predicted_times <- 0:100
  expect_equal(predict_mean(survival_curve, predicted_times), 0)
})

# predict_median ------------------------------------------------------------
testthat::test_that("predict_median functionality",{
  #Full linear survival curve
  survival_curve <- seq(1,0,length.out = 101)
  predicted_times <- 0:100
  expect_equal(predict_median(survival_curve, predicted_times), 50)

  #Linear extension
  survival_curve <- seq(1,.7,length.out = 31)
  predicted_times <- 0:30
  expect_equal(predict_median(survival_curve, predicted_times), 50)

  survival_curve <- seq(1,.3,length.out = 71)
  predicted_times <- 0:70
  expect_equal(predict_median(survival_curve, predicted_times), 50)

  #Infinite survival curve
  survival_curve <- rep(1,101)
  predicted_times <- 0:100
  expect_equal(predict_median(survival_curve, predicted_times), Inf)

  #Zero survival curve
  survival_curve <- rep(0,101)
  predicted_times <- 0:100
  expect_equal(predict_median(survival_curve, predicted_times), 0)
})

testthat::context("Testing the extra utility methods (get_param_influence,log_loss).")

# get_param_influence ------------------------------------------------------------
#Basic test for basic function (mostly to increase code coverage).
testthat::test_that("get_influence functionality",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  mod <- mtlr(formula,data)
  weights <- mod$weight_matrix
  expect_equal(unname(get_param_influence(mod)), sum(abs(weights[,2, drop=FALSE])))
})

# loglik_loss ------------------------------------------------------------

testthat::test_that("loglik_loss functionality all uncensored",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  object <- mtlr(formula,data)

  #All uncensored
  newdata <- survival::leukemia[c(1,2,4,5),]
  curves <- predict(object, newdata)

  time <- curves[,1]
  #We only need one curve because everyone is the same.
  curve <- curves[,2]
  curve_diff <- -diff(c(curve,0))
  ind <- findInterval(newdata$time, time)
  loss <- -sum(log(curve_diff[ind]+1e-05))
  expect_equal(loglik_loss(object,newdata),loss/nrow(newdata))
})

testthat::test_that("log_loss functionality all censored",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  object <- mtlr(formula,data)

  #All uncensored
  newdata <- survival::leukemia[c(3,6,9,11),]
  curves <- predict(object, newdata)

  time <- curves[,1]
  #We only need one curve because everyone is the same.
  curve <- curves[,2]
  probs <- predict_prob(curve,time,newdata$time)

  loss <- -sum(log(probs+1e-05))
  expect_equal(loglik_loss(object,newdata),loss/nrow(newdata))
})

testthat::test_that("log_loss functionality mixed uncensored and censored",{
  formula <- survival::Surv(time,status)~.
  data <- survival::leukemia
  object <- mtlr(formula,data)

  #All uncensored
  newdata <- survival::leukemia[c(1,2,4,5,3,6,9,11),]
  curves <- predict(object, newdata)

  time <- curves[,1]
  #We only need one curve because everyone is the same.
  curve <- curves[,2]
  #Uncensored
  curve_diff <- -diff(c(curve,0))
  ind <- findInterval(newdata$time[1:4], time)
  loss <- -sum(log(curve_diff[ind]+1e-05))

  #Censored
  probs <- predict_prob(curve,time,newdata$time[5:8])
  loss <- loss -sum(log(probs+1e-05))

  expect_equal(loglik_loss(object,newdata),loss/nrow(newdata))
})

testthat::test_that("log_loss functionality mixed uncensored and (left/interval/right) censored",{
  formula <- survival::Surv(time1,time2, type = "interval2")~.
  time1 <- c(NA, 4, 7, 12, 10, 6, NA, 3,5,9,10,12,NA,4,6,2,NA,16,15,11)
  time2 <- c(14, 4, 10, 12, NA, 9, 5, NA, NA, NA, NA, 15,22,4,8,6,2,20,23,11)
  set.seed(42)
  dat <- cbind.data.frame(time1, time2, importantfeature = 1)
  object <- mtlr(formula,dat,normalize = F)

  #All uncensored
  curves <- predict(object)

  time <- curves[,1]
  #We only need one curve because everyone is the same.
  curve <- curves[,2]
  #Uncensored
  curve_diff <- -diff(c(curve,0))
  ind <- findInterval(dat$time1[c(2,4,14,20)], time)
  loss <- -sum(log(curve_diff[ind]+1e-05))

  #Left Censored
  probs <- 1 - predict_prob(curve,time,dat$time2[c(1,7,13,17)])
  loss <- loss -sum(log(probs+1e-05))

  #Right Censored
  probs <- predict_prob(curve,time,dat$time1[c(5,8,9,10,11)])
  loss <- loss -sum(log(probs+1e-05))

  #Interval Censored
  probsU <- predict_prob(curve,time,dat$time1[c(3,6,12,15,16,18,19)])
  probsL <- predict_prob(curve,time,dat$time2[c(3,6,12,15,16,18,19)])
  probs <- probsU - probsL
  loss <- loss -sum(log(probs+1e-05))

  expect_equal(loglik_loss(object,dat),loss/nrow(dat))
})

---
title: "Introduction to the MTLR Workflow"
author: "Humza Haider (hshaider@ualberta.ca)"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to the MTLR Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


Here we give a brief introduction to using Multi-task Logistic Regression (MTLR) for survival prediction. Note that MTLR was specifically designed to give __survival probabilities__ across a __range of times__ for __individual observations__. This differs from models which produce risk scores (such as those given by Cox proportional hazards), single time probability models (such as the Gail model), and population wide models (*e.g.* Kaplan-Meier curves). Producing survival probabilities over a range of times gives a more holistic view of survival to patients and physicians which may be critical in making healthcare decisions. 

MTLR was introduced first in 2011 at NIPS under the name, ["Learning Patient-Specific Cancer Survival Distributions as a Sequence of Dependent Regressors"](https://papers.nips.cc/paper/4210-learning-patient-specific-cancer-survival-distributions-as-a-sequence-of-dependent-regressors). Since then much work has been done including a [website](http://pssp.srv.ualberta.ca/) which can be used to build MTLR models on uploaded data. While this is an extremely beneficial resource we have extended MTLR to be included in the R environment to make comparisons to other survival methods and use tools included in other R packages, such as `survival` and `randomForestSRC`.


MTLR can be used for survival data containing right, left, interval, or no censoring. In addition, these types of censoring can be mixed in the same dataset. Documentation on utilizing these different types of censoring can be found using `help(mtlr)`. In this vignette we will consider an example which includes right censoring only. Namely, we will be using the `lung` dataset from the `survival` package.

##Data: lung

One can access the `lung` dataset by loading the `survival` package.

```{r}
library(survival)
#Looking at the top 6 rows...
head(lung)
#help(lung) #See the basic information of lung.
```

If you look at the help file for `lung` you will see the following feature definitions:

* inst:	 Institution code
* **time**:	 Survival time in days
* **status**:	 censoring status 1=censored, 2=dead
* age:	 Age in years
* sex:	 Male=1 Female=2
* ph.ecog:	 ECOG performance score (0=good 5=dead)
* ph.karno:	 Karnofsky performance score (bad=0-good=100) rated by physician
* pat.karno:	 Karnofsky performance score as rated by patient
* meal.cal:	 Calories consumed at meals
* wt.loss:	 Weight loss in last six months

Most importantly you will notice the two features needed for every survival dataset for use of MTLR -- an event time (here `time`), 
and the indicator identifying if an observation is uncensored/censored (here `status`). For this example we have `status == 1` indicating a right censored individual and `status == 2` indicating an uncensored individual. Later on we will be using the `Surv` function to structure our survival data for MTLR -- there are other acceptable formats for the indicator feature (`status`) -- see `help(Surv)` for more information.

##Pre-processing

We will remove `inst` for this example since this is a categorical feature with 19 unique values and we would like to keep the number of features relatively small.

```{r}
lung <- lung[,-1]
```

Before progressing any further we will split our data into a training and testing set. Note that we could stratify our training/testing set by the censor status but for simplicity we skip that for now.

```{r}
numberTrain <- floor(nrow(lung)*0.8)
set.seed(42)
trInd <- sample(1:nrow(lung), numberTrain)
training <- lung[trInd,]
testing <- lung[-trInd,]
```

You may also notice that there are some missing values in the data, namely in `meal.cal` and `wt.loss` (although `ph.ecog`, `ph.karno`, and `pat.karno` also have missing values). The MTLR package does not handle missing values for users so this must be pre-processed ahead of time. If one passes in data which contains missing values anyway, all rows with missing values will be removed before model training/predictions. To remedy this problem we perform a very basic mean imputation on the dataset. Note that we use the means from the training set to impute the test set.

```{r}
#Perform imputation
trMeans <- colMeans(training,na.rm=T)
for(i in 1:ncol(training)){
  training[is.na(training[,i]), i] <- trMeans[i]
  testing[is.na(testing[,i]), i] <- trMeans[i]
}
```

##Model Training

Once the dataset has been prepared we can begin to play around with some of the functions found in the MTLR package. Most importantly we will be utilizing the `mtlr` function to train our model. There are a number of arguments that can be used by `mtlr`, though only a select few are discussed here. There are only two arguments required to train an `mtlr` model, **formula** and **data**. For **formula** we must structure our event time feature and censor indicator feature using the `Surv` function. Since we have `time` and `status` as these two features we can create our formula object:

```{r}
formula <- Surv(time,status)~.
```

The above says we will be training a model on the survival object created from `time` and `status` and using all the other features in our dataset as predictors. If we wanted to select a few features we could do this as well, for example, with `age` and `sex`.

```{r}
formulaSmall <- Surv(time, status)~age+sex
```

Next, we just need the **data** argument which in our case is ``training``. We can finally make our first model! 

```{r}
library(MTLR)
fullMod <- mtlr(formula = formula, data = training)
smallMod <- mtlr(formula = formulaSmall, data = training)
#We will print the small model so the output is more compact.
smallMod
```

There is a lot to take in at first from the output of the `mtlr` model. The first item is simply the call that was used to build the model. Next is the time points that `mtlr` used to train the model. If these time points are not specified when constructing the model then `mtlr` will choose time points based on the quantiles of the event time feature. Additionally, the number of time points is chosen to be the sqrt(N) where N is the number of observations. Since we had 205 training instances and the sqrt(205 = 14.317) `mtlr` rounded up to 15 time points. 

Last, `mtlr` outputs the weight matrix for the model -- these are the weights corresponding to each feature at each time point (additionally notice that we include the bias weights). The row names correspond to the time point for which the feature weight belongs. If you would like to access these weights, they are saved in the model object as `weight_matrix` so you can access them using `smallMod$weight_matrix`.

We can also plot the weights for a `mtlr` model. Before we printed the small model but here we will look at the weights for the complete model.


```{r, fig.width=7, fig.align="center"}
plot(fullMod)
```


By default, `plot` will only look at the 5 features which had the largest sum of absolute values across time (the most influence). You can alter these specifications by playing with the arguments in `plot`.

##Model Predictions

Now that we have trained a MTLR model we should make some predictions! This is where our `testing` set and the `predict` function will come into play. Note that there are a number of predictions we may be interested in acquiring. First, we may want to view the survival curves of our test observations.

```{r}
survCurves <- predict(fullMod, testing, type = "survivalcurve")
#survCurves is pretty large so we will look at the first 5 rows/columns.
survCurves[1:5,1:5]
```

When we use the `predict` function for survival curves we will be returned a matrix where the first column (time) is the list of time points that the model evaluated the survival probability for each observation (these will be the time points used by `mtlr` and an additional 0 point). Every following column will correspond to the row number of the data passed in, *e.g.* column 2 (named 1) corresponds to row 1 of `testing`. Each row of this matrix gives the probabilities of survival at the corresponding time point (given by the time column). For example, testing observation 1 has a survival probability of 0.919 at time 60.625. 

Since these curves may be hard to digest by observing a matrix of survival probabilities we can also choose to plot them.

```{r, fig.width=7, fig.height=5, fig.align="center"}
plotcurves(survCurves, 1:10)
```


Here we have specified that we want to observe the survival curves for the first 10 observations (corresponding to the first 10 rows of `testing`). You will notice that these curves have been smoothed whereas before we only had probabilities for certain time points. We have performed a monotonic spline fit to those survival probabilities to produce the curves you see here. 

Additionally, you may have specific plot specifications you want to make. `plotcurves` is simply returning a `ggplot2` object so specifications can be made like you would make to any other `ggplot2` graphic. For example, `plotcurves(survCurves, 1:10) + ggplot2::xlab("Days")` would change the x-axis label to "Days" instead of "Time".

###Mean/Median Survival Time 

In addition to the entire survival curve one may also be interested in the average survival time. This is again available from the `predict` function.

```{r}
#Mean
meanSurv <- predict(fullMod, testing, type = "mean_time")
head(meanSurv)
#Median
medianSurv <- predict(fullMod, testing, type = "median_time")
head(medianSurv)
```

Here the mean survival time corresponds to the area under the survival curve of each observation. One subtlety is that many survival curves never touch zero probability making this area not well-defined. When this occurs, a linear fit is drawn from the time = 0, survival probability = 1 point to the last time point and extended to the 0 probability time. For example, below we have drawn a linear extension on the curves below to calculate the mean survival time.

```{r, echo=FALSE, out.width = '90%', fig.align="center"}
knitr::include_graphics("linearExtension.png")
```

This is also performed when calculating the median survival time if the last survival probability is above 0.5. 

##Survival Probability at Event Time

The last prediction type supported is acquiring the observations survival probability at the respective event time. However, in order to use this prediction, the event time (whether censored or uncensored) must be included in the features passed into the `predict` function. 

```{r}
survivalProbs <- predict(fullMod, testing, type = "prob_event")
head(survivalProbs)
#To see what times these probabilities correspond to:
head(testing$time)
```

You will notice that some of these survival probabilities correspond to 0 (usually those with very large event times). We again have drawn the linear extension for the survival time if the event time could not be mapped onto the survival curve.

##Miscellaneous Commands
###`mtlr_cv`
Previously we just used the default settings of `mtlr`. However, a number of things can be adjusted included the number of time points, the exact time points used, the initialization of the feature weights, and the regularization parameter (C1) which corresponds to the C1 given in the [NIPS paper](https://papers.nips.cc/paper/4210-learning-patient-specific-cancer-survival-distributions-as-a-sequence-of-dependent-regressors). The `mtlr_cv` function helps to select a value of C1. Given a vector of values to test for C1, `mtlr_cv` will do internal cross validation to select the optimal C1 for some criteria. Currently the only optimization is referred to as the log-likelihood loss (see the "Details" section of `help(mtlr_cv)`). For example, we use this command with 5 values of C1 (although there is a default of (0.001,0.01,0.1,1,10,100,1000)).

```{r}
mtlr_cv(formula,training, C1_vec = c(0.01,0.1,1,10,100))
```

The output gives us the best value of C1 and the losses for the values tested. Once we have the best value of C1 we can then use the `mtlr` function with the chosen value of C1. 

###`create_folds`

As we mentioned, `mtlr_cv` uses an internal k-fold cross validation to evaluate the loss. We also export the function (`create_folds`) used to create these cross-validation folds as it is creating folds in a unique way.

These folds can be deterministic, semi-deterministic, or totally random. The deterministic folds arise by stratifying folds by censor status and attempting to create equal ranges in the event times within each fold. This is done by first stratifying the survival dataset into a censored and uncensored portion and then sorting each portion by the event time. These portions are then numbered off into k different folds (see figure below). This option corresponds to "fullstrat".

```{r, echo=FALSE, out.width = '90%',fig.cap='The "fullstrat" method of cross-fold validation. Here "delta" refers to the feature used to identify censored/uncensored observations.', fig.align="center"}
knitr::include_graphics("crossvalidation.png")
```

The semi-deterministic method stratifies the folds by censor status but ignores event time. In the figure above this would be the same process but skipping the sorting of time. This option is called "censorstrat". The completely random method ignores the censor status and the event time and randomly assigns observations to folds -- the "random" option.

We include the `create_folds` function in the event one would want to perform k-fold cross-validation on their dataset instead of using a training/testing split. For example, one could set up for 5-fold cross-validation like so:

```{r}
#Recall we are using the lung dataset.
testInd <- create_folds(lung$time, lung$status, nfolds = 5, foldtype = "fullstrat")
```

Now `testInd` is a list of length 5 where each of the 5 items are the indices of the test set to be used for each fold. One could now use a `for` loop and iterate over 5-folds and index `lung` by the test indices  (`lung[testInd,]`) for the test set and all but the test indices (`lung[-testInd,]`) for the training set and train these 5 models.

##Evaluation

The MTLR package does not directly supply functions on how to evaluate the survival curves, but instead the methods to produce these curves. For a detailed discussion on a variety of evaluation metrics and access to some R-scripts containing evaluation code please see the paper [Effective Ways to Build and Evaluate Individual Survival Distributions](https://arxiv.org/abs/1811.11347).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method.R
\name{print.mtlr}
\alias{print.mtlr}
\title{Printing an MTLR object.}
\usage{
\method{print}{mtlr}(x, digits = max(options()$digits - 4, 3), ...)
}
\arguments{
\item{x}{an object of class mtlr (result from calling \code{\link[MTLR]{mtlr}}).}

\item{digits}{The number of digits to print mtlr weights.}

\item{...}{for future methods.}
}
\value{
Call, the original call to the mtlr function. Time points, the time points selected by the mtlr model. Weights, the weights of
each feature across time -- rows represent each time point and each column corresponds to a feature.
}
\description{
Print an object created by \code{\link[MTLR]{mtlr}}.
}
\seealso{
\code{\link[MTLR]{mtlr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{create_folds}
\alias{create_folds}
\title{Create folds for cross-validation.}
\usage{
create_folds(
  time,
  delta,
  nfolds,
  foldtype = c("fullstrat", "censorstrat", "random")
)
}
\arguments{
\item{time}{a vector of event times.}

\item{delta}{a vector of indicators for uncensored/censored data. The type of censoring here is not considered so it is suggested this function not
be used for data with mixed censoring types. The specific indicator value does not matter as long as censored and uncensored observations have different
values for their indicator.}

\item{nfolds}{The number of folds to create.}

\item{foldtype}{type of cross validation folds. Full stratification, "fullstrat", sorts observations by their event time and their event indicators
and numbers them off into folds. This effectively give each fold approximately the same number of uncensored observations as well as keeps the range
of time points as equivalent as possible across folds. This type of cross-validation is completely deterministic.
Censored stratification, "censorstrat", will put approximately the same number of uncensored observations in each fold but not pay any attention to
event time. This is partially stochastic. The totally random cross-validation, "random", randomly assigns observations to folds without considering
event time nor event status.}
}
\value{
a list of size nfolds where each list component contains the indices of the test data for each fold.
}
\description{
Create the test folds for k-fold cross validation. These cross-validation types differ from typical stratified cross-validation as this function also
considers the range of event times in the data.
}
\seealso{
\code{\link[MTLR]{mtlr_cv}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method.R
\name{predict.mtlr}
\alias{predict.mtlr}
\title{Predictions for MTLR}
\usage{
\method{predict}{mtlr}(
  object,
  newdata,
  type = c("survivalcurve", "prob_times", "prob_event", "mean_time", "median_time"),
  add_zero = T,
  times = c(),
  ...
)
}
\arguments{
\item{object}{an object of class mtlr, generated by the \link[MTLR]{mtlr}.}

\item{newdata}{an optional new dataframe for which to perform predictions using MTLR. If left empty, predictions will
be performed using the dataset used to generate the original mtlr object -- note that any error calculation on these
predictions will be optimistic since this will only be the resubstitution error and not be representative of error on a new test set.}

\item{type}{the type of prediction desired. Options are the survival curve for the time points selected by mtlr ("survivalcurve"), the
survival curve for given times ("prob_times"), the probability of survival at the observations event time ("prob_event"),
the mean survival time ("mean_time"), and the median survival time ("median_time").

For "survivalcurve" and "prob_times", the first column of the matrix returned will correspond to the time points and all other columns will be
the observations survival probability at those associated time points. The index of a (row) observation in newdata will correspond
to the \emph{ith + 1} column of the returned matrix.

If "prob_event" is chosen the response (event time) is required. For both "prob_event" and "prob_times", if the event time is larger
than all of the time points used to build the mtlr model then the last (lowest) probability is used. For example, if the event time is
100 but the largest time point estimated by the mtlr model was 80 then the survival probability at 100 is equal to the survival
probability at 80, \emph{i.e. S(100) = S(80)}.

For "mean_time", if survival curves do not extend to zero survival
probability a linear extension is added (a linear line from (time = 0,probability = 1) to (time = ?, probability =0)). This is the
same for "median_time" except the line need only extend to survival probability = 0.5.
A mean/median survival time of Inf is returned for survival curves with all survival probabilities of 1.}

\item{add_zero}{if TRUE, a time point of "0" and a survival probability of "1" will be added to all survival curves. Additionally,
if add_zero is TRUE, type = "mean_time" will represent the average survival time overall but if FALSE, then "mean_time" will be reduced
by roughly the value of the first time point. However, "median_time" and "prob_event" will be unchanged.}

\item{times}{For prediction method "prob_times" you may specify the times at which to predict the survival probability for each row in newdata.
This values defaults to all unique event times (both censored and uncensored) in the data on which the model was trained.}

\item{...}{for future methods.}
}
\value{
The desired prediction type (a matrix or vector of predictions).
}
\description{
Compute survival curves and other fitted values for a model generated by \code{\link[MTLR]{mtlr}}.
}
\note{
The predictions generated by type = "survivalcurve" can be plotted using \code{\link[MTLR]{plotcurves}} -- packages
ggplot2 and reshape2  must be installed to use this function.
}
\examples{
library(survival)
mod <- mtlr(Surv(time,status)~., data = lung)

#Here our predictions are on the data from which we trained so our results will be optimistic
# since they are produced from resubstitution as opposed to some new test set.
predict(mod, type = "survivalcurve")
predict(mod, type = "prob_event")
predict(mod, type = "median_time")
predict(mod, type = "mean_time")

#Notice the difference of about 59:
predict(mod, type = "mean_time", add_zero = FALSE)
}
\seealso{
\code{\link[MTLR]{mtlr}} \code{\link[MTLR]{plotcurves}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mtlr_cv.R
\name{mtlr_cv}
\alias{mtlr_cv}
\title{MTLR Internal Cross-Validation for Selecting C1.}
\usage{
mtlr_cv(
  formula,
  data,
  time_points = NULL,
  nintervals = NULL,
  normalize = T,
  C1_vec = c(0.001, 0.01, 0.1, 1, 10, 100, 1000),
  train_biases = T,
  train_uncensored = T,
  seed_weights = NULL,
  previous_weights = T,
  loss = c("ll", "concordance"),
  nfolds = 5,
  foldtype = c("fullstrat", "censorstrat", "random"),
  verbose = FALSE,
  threshold = 1e-05,
  maxit = 5000,
  lower = -15,
  upper = 15
)
}
\arguments{
\item{formula}{a formula object with the response to the left of the "~" operator. The response must be a survival object returned
by the \code{\link[survival]{Surv}} function.}

\item{data}{a data.frame containing the features for survival prediction. These must be variables corresponding to the formula object.}

\item{time_points}{the time points for MTLR to create weights. If left as NULL, the time_points chosen will be based on equally spaced quantiles
of the survival times. In the case of interval censored data note that only the start time is considered and not the end time for selecting time points.
It is strongly recommended to specify time points if your data is heavily interval censored. If time_points is not NULL then nintervals is ignored.}

\item{nintervals}{Number of time intervals to use for MTLR. Note the number of time points will be nintervals + 1. If left as NULL
a default of sqrt(N) is used where N is the number of observations in the supplied dataset. This parameter is ignored if time_points is specified.}

\item{normalize}{if TRUE, variables will be normalized (mean 0, standard deviation of 1). This is STRONGLY suggested. If normalization
does not occur it is much more likely that MTLR will fail to converge. Additionally, if FALSE consider adjusting "lower" and "upper"
used for L-BFGS-B optimization.}

\item{C1_vec}{a vector of regularization parameters to test. All values must be non-negative. For large datasets you may want to reduce the number
of value tried to increase efficiency. Similarly for nfolds.}

\item{train_biases}{if TRUE, biases will be trained before feature weights (and again trained while training feature weights). This
has shown to speed up total training time.}

\item{train_uncensored}{if TRUE, one round of training will occur assuming all event times are uncensored. This is done due to the non-convexity issue
that arises in the presence of censored data. However if ALL data is censored we recommend setting this option to FALSE as it has shown to give poor
results in this case.}

\item{seed_weights}{the initialization weights for the biases and the features. If left as NULL all weights are initialized to zero. If seed_weights are
specified then either nintervals or time_points must also be specified. The length of seed_weights should correspond to (number of features + 1)*(length of
time_points) = (number of features + 1)*(nintervals + 1).}

\item{previous_weights}{a boolean specifying if sequential folds should use the previous fold's parameters as seed_weights. Doing this will likely
speed up the computation time for cross-validation as we are providing weights which are (likely) close to the optimal weights. Note that this is
done separately for each value of C1 so there is no parameter sharing between different values of C1, and instead only across the same value of C1.}

\item{loss}{a string indicating the loss to optimize for which to choose the regularization parameter. Currently one can optimize for the log-likelihood ("ll")
or concordance ("concordance"). See details regarding these losses.}

\item{nfolds}{the number of internal cross validation folds, default is 5.}

\item{foldtype}{type of cross validation folds. Full stratification, "fullstrat", sorts observations by their event time and their event indicators
and numbers them off into folds. This effectively give each fold approximately the same number of uncensored observations as well as keeps the range
of time points as equivalent as possible across folds. This type of cross-validation is completely deterministic.
Censored stratification, "censorstrat", will put approximately the same number of uncensored observations in each fold but not pay any attention to
event time. This is partially stochastic. The totally random cross-validation, "random", randomly assigns observations to folds without considering
event time nor event status.}

\item{verbose}{if TRUE the progress will be printed for every completed value of C1.}

\item{threshold}{The threshold for the convergence tolerance (in the objective function) when training the feature weights.
This threshold will be passed to \link[stats]{optim}.}

\item{maxit}{The maximum iterations to run for MTLR. This parameter will be passed to \link[stats]{optim}.}

\item{lower}{The lower bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.}

\item{upper}{The upper bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.}
}
\value{
Performing mtlr_cv will return the following:
\itemize{
  \item best_C1: The value of C1 which achieved the best (lowest) loss.
  \item avg_loss: The averaged value of loss across the five folds for each value of C1 tested.
  }
}
\description{
MTLR Internal Cross-Validation for Selecting C1.
}
\details{
The log-likelihood loss and concordance are supported for optimizing C1. Here the log-likelihood loss considers censored and uncensored observations differently.
For uncensored observations, we assign a loss of the negative log probability assigned to the interval in which the observation had their event, \emph{e.g.}
if an observation had a 20% chance of having its event between timepoint1 and timepoint2 and it did have it's event in that interval then the loss
is -log(0.2). We want these probabilities to be large so we would normally want to maximize this value (since logs of probabilities are negative)
but we take the negative and instead minimize the value, thus we want the lowest loss. For censored observations we take the log of the probability
of survival at the time of censoring, \emph{e.g.} if an observation is censored at time = 42 we take the negative log of the survival probability assigned
to time 42 as the loss.

For the concordance loss, C1 is chosen to maximize the overall concordance when using the negative median as the "risk" score. This is completed using survConcordance in the survival package.
}
\examples{
library(survival)
cv_mod <- mtlr_cv(Surv(time,status)~., data = lung)
#Note the best C1 also corresponds to the lost average loss:
cv_mod
}
\seealso{
\code{\link[MTLR]{mtlr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotcurves.R
\name{plotcurves}
\alias{plotcurves}
\title{Graphically Visualize MTLR Survival Curves}
\usage{
plotcurves(curves, index = 1, color = c(), xlim = c(), remove_legend = TRUE)
}
\arguments{
\item{curves}{survival curves formatted the same as those from predict.mtlr. Time points must be in the first column of the matrix followed by
columns representing survival probabilities for each observation.}

\item{index}{the index of the observation to plot. Here an index of 1 will refer to the second column of the curves object. If over 15 indices are
given the legend will be removed as to not take up plotting space. To avoid this behavior set remove_legend = FALSE.}

\item{color}{the color of the plotted survival curve. The length of color must match the length of index.}

\item{xlim}{the limits of the x-axis (must be a 2 length vector).}

\item{remove_legend}{if TRUE the legend will be removed if over 15 indices are supplied. If FALSE the legend will remain, however be aware that the
legend may take up lots of space.}
}
\description{
Plot the survival curves returned from predict.mtlr. Users must have packages ggplot2 and reshape2 installed in order to use this function.
Survival curves for MTLR are smoothed using a monotonic cubic spline using a Hyman filtering between time points. For details regarding this
smoothing function see \code{\link[stats]{splinefun}}.
}
\examples{
#Set up the example:
library(survival)
mod <- mtlr(Surv(time,status)~., data = lung)
curves <- predict(mod, type = "survivalcurve")

plotcurves(curves, 1:10)
plotcurves(curves, 1:3, color = c("red","blue","purple"))
plotcurves(curves, 1:10, xlim = c(0,42))

#Note the legend is now gone:
plotcurves(curves, 1:20)

#and it is back again:
plotcurves(curves, 1:20, remove_legend = FALSE)
}
\seealso{
\code{\link[MTLR]{mtlr}} \code{\link[MTLR]{predict.mtlr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method.R
\name{plot.mtlr}
\alias{plot.mtlr}
\title{Graphical Representation of Feature Weights}
\usage{
\method{plot}{mtlr}(x, numfeatures = 5, featurenames = c(), digits, ...)
}
\arguments{
\item{x}{an object of class mtlr (result from calling \code{\link[MTLR]{mtlr}}).}

\item{numfeatures}{the number of weight to plot. Default is 5. The most influential features are chosen first.}

\item{featurenames}{the names of the specific weight to plot. These should correspond to the names
in x$weight_matrix. If featurenames are supplied, then numfeatures is ignored.}

\item{digits}{the number of digits to round to for the value of the time points.}

\item{...}{for future methods}
}
\description{
Plot the weights of an mtlr object. If packages ggplot2 and reshape2 are
not installed, a bargraph of feature \emph{influence} is given where influence is defined as the sum of absolute values of the
feature weights across time. If ggplot2 and reshape2 are installed then a plot of feature weight across time is given.
}
\examples{
#These examples are geared towards users who have installed ggplot2 and reshape2.
library(survival)
mod <- mtlr(Surv(time,status)~., data = lung)
#Basic plot with 5 most influential features
plot(mod)
#Plot all 8 features
plot(mod, numfeatures = 8)
#Suppose we want to see specifically the "meal.cal" and "ph.karno" features:
plot(mod, featurenames = c("meal.cal", "ph.karno"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mtlr.R
\name{mtlr}
\alias{mtlr}
\title{Train a Multi-Task Logistic Regression (MTLR) Model}
\usage{
mtlr(
  formula,
  data,
  time_points = NULL,
  nintervals = NULL,
  normalize = T,
  C1 = 1,
  train_biases = T,
  train_uncensored = T,
  seed_weights = NULL,
  threshold = 1e-05,
  maxit = 5000,
  lower = -15,
  upper = 15
)
}
\arguments{
\item{formula}{a formula object with the response to the left of the "~" operator. The response must be a survival object returned
by the \code{\link[survival]{Surv}} function.}

\item{data}{a data.frame containing the features for survival prediction. These must be variables corresponding to the formula object.}

\item{time_points}{the time points for MTLR to create weights. If left as NULL, the time_points chosen will be based on equally spaced quantiles
of the survival times. In the case of interval censored data note that only the start time is considered and not the end time for selecting time points.
It is strongly recommended to specify time points if your data is heavily interval censored. If time_points is not NULL then nintervals is ignored.}

\item{nintervals}{Number of time intervals to use for MTLR. Note the number of time points will be nintervals + 1. If left as NULL
a default of sqrt(N) is used where N is the number of observations in the supplied dataset. This parameter is ignored if time_points is specified.}

\item{normalize}{if TRUE, variables will be normalized (mean 0, standard deviation of 1). This is STRONGLY suggested. If normalization
does not occur it is much more likely that MTLR will fail to converge. Additionally, if FALSE consider adjusting "lower" and "upper"
used for L-BFGS-B optimization.}

\item{C1}{The L2 regularization parameter for MTLR. C1 can also be selected via \code{\link[MTLR]{mtlr_cv}}. See "Learning Patient-Specific Cancer Survival Distributions as a Sequence of Dependent
Regressors" by Yu et al. (2011) for details.}

\item{train_biases}{if TRUE, biases will be trained before feature weights (and again trained while training feature weights). This
has shown to speed up total training time.}

\item{train_uncensored}{if TRUE, one round of training will occur assuming all event times are uncensored. This is done due to the non-convexity issue
that arises in the presence of censored data. However if ALL data is censored we recommend setting this option to FALSE as it has shown to give poor
results in this case.}

\item{seed_weights}{the initialization weights for the biases and the features. If left as NULL all weights are initialized to zero. If seed_weights are
specified then either nintervals or time_points must also be specified. The length of seed_weights should correspond to (number of features + 1)*(length of
time_points) = (number of features + 1)*(nintervals + 1).}

\item{threshold}{The threshold for the convergence tolerance (in the objective function) when training the feature weights.
This threshold will be passed to \link[stats]{optim}.}

\item{maxit}{The maximum iterations to run for MTLR. This parameter will be passed to \link[stats]{optim}.}

\item{lower}{The lower bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.}

\item{upper}{The upper bound for L-BFGS-B optimization. This parameter will be passed to \link[stats]{optim}.}
}
\value{
An mtlr object returns the following:
\itemize{
  \item weight_matrix: The matrix of feature weights determined by MTLR.
  \item x: The dataframe of features (response removed). Note observations with missing values will have been removed (this is the dataset on which
  MTLR was trained).
  \item y: The matrix of response values MTLR uses for training. Each column corresponds to an observation and rows as time points. A value of 1
  indicates a observation was either censored or had their event occur by that time.
  \item response: The response as a Surv object (specified by formula).
  \item time_points: The timepoints selected and used to train MTLR.
  \item C1: The regularization parameter used.
  \item Call: The original call to mtlr.
  \item Terms: The x-value terms used in mtlr. These are later used in \code{\link[MTLR]{predict.mtlr}}
  \item scale: The means and standard deviations of features when normalize = TRUE. These are used in \code{\link[MTLR]{predict.mtlr}}. Will be
  NULL if normalize = FALSE.
  \item xlevels: The levels of the features used. This is used again by \code{\link[MTLR]{predict.mtlr}}.
}
}
\description{
Trains a MTLR model for survival prediction. Right, left, and interval censored data are all supported.
}
\details{
This function allows one to train an MTLR model given a dataset containing survival data. mtlr uses the Limited-Memory
Broyden–Fletcher–Goldfarb–Shanno (L-BFGS-B) approximation method to train feature weights. This training is outsourced to the internal
\link[stats]{optim} function in R. Currently only a few parameters (namely threshold, maxit,lower, upper) of optim are supported, more will
likely become available in the future.

Weights are initialized to 0 prior to training. Under default settings, the bias weights
will be trained before considering feature weights. As Yu et al. (2011) specified, the introduction of censored observations creates a non-convex
loss function. To address this, weights are first trained assuming all event times are \emph{uncensored}. Once these starting weights have
been trained another round of training is performed using the true values of the event indicator (censored/uncensored). However, in the event of
all censored data this has shown to negatively effect the results. If all data is censored (either left, right, or interval2) we suggest setting
train_uncensored = FALSE.

Yu et al. (2011) actually suggested two regularization parameters, C1 to control the size of the feature weights and C2 to control the smoothness.
In Ping Jin's masters thesis (Using Survival Prediction Techniques to Learn Consumer-Specific Reservation Price Distributions) he showed that C2
is not required for smoothness and C1 will suffice (Appendix A.2) so we do not support the C2 parameter in this implementation.

If an error occurs from optim it is likely the weights are getting too large. Including fewer time points (or specifying better time points) in
addition to changing the lower/upper bounds of L-BFGS-B may resolve these issues. The most common failure has been that the objective value sees
infinite values due to extremely large feature weights.

\strong{Censored data:} Right, left, and interval censored data are all supported both separately and mixed. The convention to input these types of
data follows the \link[survival]{Surv} object format.
Per the Surv documentation, "The [interval2] approach is to think of each observation as a time interval with (-infinity, t) for left censored,
(t, infinity) for right censored, (t,t) for exact and (t1, t2) for an interval. This is the approach used for type = interval2.
Infinite values can be represented either by actual infinity (Inf) or NA." See the examples below for an example of inputting this type of data.
}
\examples{
#Access the Surv function and the leukemia/lung dataset.
library(survival)
simple_mod <- mtlr(Surv(time,status)~., data = leukemia)
simple_mod

bigger_mod <- mtlr(Surv(time,status)~., data = lung)
bigger_mod

#Note that observations with missing data were removed:
nrow(lung)
nrow(bigger_mod$x)


# Mixed censoring types
time1 = c(NA, 4, 7, 12, 10, 6, NA, 3) #NA for right censored
time2 = c(14, 4, 10, 12, NA, 9, 5, NA) #NA for left censored
#time1 == time2 indicates an exact death time. time2> time1 indicates interval censored.
set.seed(42)
dat = cbind.data.frame(time1, time2, importantfeature = rnorm(8))
formula = Surv(time1,time2,type = "interval2")~.
mixedmod = mtlr(formula, dat)

}
\seealso{
\code{\link[MTLR]{predict.mtlr}} \code{\link[MTLR]{mtlr_cv}} \code{\link[MTLR]{plot.mtlr}}  \code{\link[MTLR]{plotcurves}}
}
